package Hawat::Controller::DBItem;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Data::Dumper;
use Mentat::Core qw(:c_hawat);

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

BEGIN { extends 'Hawat::Controller::Base'; }

# Definitions of action configurations
use constant CNF_ACTION       => 'action';
use constant CNF_ATTRIBUTE    => 'attribute';
use constant CNF_BACKLINK     => 'backlink';
use constant CNF_BACKLINK_D   => 'backlink_d';
use constant CNF_CONTROLLER   => 'controller';
use constant CNF_ITEM         => 'item';
use constant CNF_ITEM_S       => 'item_s';
use constant CNF_SUBITEM      => 'subitem';
use constant CNF_ITEMS        => 'items';
use constant CNF_ITEMS_S      => 'items_s';
use constant CNF_SUBITEMS     => 'subitems';
use constant CNF_LIMIT        => 'limit';
use constant CNF_MENU         => 'menu';
use constant CNF_MODEL        => 'model';
use constant CNF_OPERATION    => 'operation';
use constant CNF_MULTI        => 'multi';
use constant CNF_MSG_FAILURE  => 'msg_failure';
use constant CNF_MSG_NONE     => 'msg_none';
use constant CNF_MSG_SUCCESS  => 'msg_success';
use constant CNF_REGEXP_ID    => 'regexp_id';
use constant CNF_SEARCH       => 'search';
use constant CNF_SORT         => 'sort';
use constant CNF_SORTBY       => 'sortby';
use constant CNF_FILTER       => 'filter';
use constant CNF_QUICK        => 'quick';
use constant CNF_ADVANCED     => 'advanced';
use constant CNF_DFROM        => 'dfrom';

# Valid action configurations in hash format for validations
my %VALID_CONFIGS = (
        CNF_ACTION()      => 1,
        CNF_ATTRIBUTE()   => 1,
        CNF_BACKLINK()    => 1,
        CNF_BACKLINK_D()  => 1,
        CNF_CONTROLLER()  => 1,
        CNF_ITEM()        => 1,
        CNF_ITEM_S()      => 1,
        CNF_SUBITEM()     => 1,
        CNF_ITEMS()       => 1,
        CNF_ITEMS_S()     => 1,
        CNF_SUBITEMS()    => 1,
        CNF_LIMIT()       => 1,
        CNF_MENU()        => 1,
        CNF_MODEL()       => 1,
        CNF_OPERATION()   => 1,
        CNF_MULTI()       => 1,
        CNF_MSG_FAILURE() => 1,
        CNF_MSG_NONE()    => 1,
        CNF_MSG_SUCCESS() => 1,
        CNF_REGEXP_ID()   => 1,
        CNF_SEARCH()      => 1,
        CNF_SORT()        => 1,
        CNF_SORTBY()      => 1,
        CNF_FILTER()      => 1,
        CNF_QUICK()       => 1,
        CNF_ADVANCED()    => 1,
        CNF_DFROM()       => 1,
    );

=head1 NAME

Hawat::Controller::DBItem - Catalyst controller implementing core CRUD operations with arbitrary database items

=head1 DESCRIPTION

This catalyst controller contains actions implementing the core CRUD operations
with arbitrary database items. Available actions are listed below. Every action takes
care of argument validation and redirects user to appripriate error pages. Item operations
without content like delete or modify also take care of operation confirmation process and
user redirection to previous page, after the operation was performed. Of course, with
appropriate status message displayed to the user.

Every action should be called via the $c->forward() method, for example:

    $c->forward('/dbitem/fetch',[$conf, $id, @args]);

I<$conf> is the special configuration hash, which configures and controls the behaviour
of the action (see section L</"ACTION CONFIGURATIONS"> for details).

=head1 ACTION CONFIGURATIONS

Each actions takes special HASH REFERENCE I<$conf> as first argument, which augments
certain aspects of action operation flow. This configuration may contain some of the
following configuration keys, depending on the action. Please see the description of
appropriate action in section L</"ACTIONS"> for list of mandatory and optional configurations.

=head2 backlink

URL location to which to redirect after the action is complete.

=head2 backlink_d

URL location to which to redirect after the item is successfully deleted from database.

=head2 controller

Name of the controller, that is handling the actions.

=head2 item

Name of the database item in singular form (eg. user, role, etc.). Case insensitive.

=head2 items

Name of the database items in plural form (eg. users, roles, etc.). Case insensitive.

=head2 limit

Limit the size of the result for find method.

=head2 menu

Name of the appropriate menu item to be activated.

=head2 model

Name of the model, which is handling the actual processing. The model must have
following interface:

=over

=item C<my ($items, $count, $count_all, $duration, $query, @more) = $model-E<gt>find($search, $limit, $skip, $sortby, $sort, @args)>

Find items in database according to the filters, additional arguments are optional
and model dependent. This method must return fetched items in I<$items>, item count
in result in I<$count>, total item count in I<$count_all>, operation duration in
I<$duration> and executed query in I<$query> return values. In I<@more>, it is possible
to return additional objects. Returned values will then be present in stash at following
locations:

 "${items}"           => $items     # For $items = 'roles' it will be "roles", not "items" !!!
 "${items}_count"     => $count     # Similar as above
 "${items}_count_all" => $count_all # Similar as above
 "${items}_duration"  => $duration  # Similar as above
 "${items}_query"     => $query     # Similar as above
 "${items}_more"      => \@more     # Similar as above

Additionally, following variables will be set:

 "${items}_search"    # Search argument
 "${items}_paginator" # Result paginator

=item C<my ($item, $count, $duration, @more) = $model-E<gt>fetch($id, @args)>

Fetch item with given ID [MANDATORY], additional arguments are optional and model
dependent. This method must return fetched item in I<$item>, item count in I<$count>
and operation duration in I<$duration> return values. In I<@more>, it is possible
to return additional objects. Returned values will then be present in stash at
following locations:

 "${item}_id"       => $id        # For $item = 'role' it will be "role_id", not "item_id" !!!
 "${item}"          => $item      # Similar as above
 "${item}_count"    => $count     # Similar as above
 "${item}_duration" => $duration  # Similar as above
 "${item}_more"     => \@more     # Similar as above

=item C<my ($item_id, $errors) = $model-E<gt>create($form, @args)>

Create new item from given form data [MANDATORY], additional arguments are optional
and model dependent. This method must return STRING $item_id as first item in the
result on SUCCESS, or undef and list of errors in ARRAY REFERENCE on FAILURE.

=item C<my ($rv, $errors) = $model-E<gt>update($id, $object, $form, @args)>

Update existing item with given form data [MANDATORY], additional arguments are optional
and model dependent. If the form data were correct, this method must return HASH REFERENCE
containing structure with following fields:

 # SUCCESS: { 'ok' => '1', 'n' => 1, 'connectionId' => 270, 'err' => undef }
 # FAILURE: { 'ok' => '1', 'n' => 0, 'connectionId' => 270, 'err' => undef }

If the form data were not correct, this method must return undef in place of $rv,
followed by list of errors in ARRAY REFERENCE.

=item C<my $item_id = $model-E<gt>rename($id, $new_id, @args)>

Rename primary identifier of given item [MANDATORY], additional arguments are optional
and model dependent. This method must return STRING $new_id as first item in the
result on SUCCESS, or undef and list of errors in ARRAY REFERENCE on FAILURE.

=item C<my $rv = $model-E<gt>delete($id, @args)>

Delete item with given ID [MANDATORY], additional arguments are optional and model
dependent. This method must return HASH REFERENCE containing structure with
following fields:

 # SUCCESS: { 'ok' => '1', 'n' => 1, 'connectionId' => 270, 'err' => undef }
 # FAILURE: { 'ok' => '1', 'n' => 0, 'connectionId' => 270, 'err' => undef }

=back

=head2 modification

Name of the method within the model which to use for object modification.

=head2 msg_failure

Message to be displayed in case of failure.

=head2 msg_none

Message to be displayed in case of no items were modified by operation.

=head2 msg_success

Message to be displayed in case of success.

=head2 multi

Switch marking operation to be working with multiple records.

=head2 regexp_id

Regular expression against which to check the format of item identifier.

=head2 search

Searching argument, will be passed down unaltered to the model.

=head2 sort

Sorting order ['asc'|'desc']

=head2 sortby

Sorting field (proper values are model dependent).

=head1 ACTIONS

=cut

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

=head2 find CONF [ARG, ...]

Find items in database according to the given rules

B<Arguments:>

=over

=item I<CONF> [HASH REFERENCE, MANDATORY]

Configuration hash, which controls inner behaviour of the action. This action accepts
following configurations (please see section L</"ACTION CONFIGURATIONS"> for descriptions):

C<items[O], limit[O], menu[O], model[M], search[O], sortby[O], sort[O]>

Note: [M] = mandatory configuration, [O] = optional configuration

=item I<ARGS> [MIXED, OPTIONAL]

Optional additional arguments, will be passed directly to the models I<find()> method

=back

=cut

sub find :Private {
    my ($self, $c, $conf, @args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Perform configuration check and normalization
    $self->_chcnf($c, $conf, CNF_MODEL);

    $conf->{CNF_OPERATION()} = $conf->{CNF_OPERATION()} || 'find';

    # Check, that the model can perform the requested operation
    my ($model, $operation) = $self->_choper($c, HAWAT_ERROR_HANDLER, $conf);

    my $quick = ($conf->{CNF_QUICK()})?1:0;

    my $dbquery = {}; my ($sortby, $sort, $limit, $page, $skip);

    # Setup the simple find arguments
    unless ($conf->{CNF_ADVANCED()}) {
        my $search = $c->request->params->{search}   || $conf->{CNF_SEARCH()} || undef;
        my $from   = $c->request->params->{datefrom} || undef;
        my $to     = $c->request->params->{dateto}   || undef;
        # Prepare the dbquery for database driver
        $dbquery->{search} = $search;
        $dbquery->{from}   = $from if $from;
        $dbquery->{to}     = $to   if $to;
        # Optionally apply the default lower boundary
        $dbquery->{from} = $conf->{CNF_DFROM()} if $conf->{CNF_DFROM()} and not $dbquery->{from};

        # Put the variables back to the stash for the views to display them
        $c->stash($conf->{CNF_ITEMS_S()}.'_search'    => $dbquery->{search},
                  $conf->{CNF_ITEMS_S()}.'_from'      => $dbquery->{from},
                  $conf->{CNF_ITEMS_S()}.'_to'        => $dbquery->{to},
        );
    }
    # Setup the advanced find arguments
    else {
        $dbquery = $conf->{CNF_ADVANCED()};
        foreach my $k (keys %$dbquery) {
            $c->stash($k => $dbquery->{$k}) if defined $dbquery->{$k};
        }
        $c->stash($conf->{CNF_ITEMS_S()}.'_search' => $dbquery);
    }

    # Prepare sorting arguments
    $sortby = $c->request->params->{sortby}   || $conf->{CNF_SORTBY()} || undef;
    $sort   = $c->request->params->{sort}     || $conf->{CNF_SORT()}   || undef;

    # Prepare limiting arguments
    $limit  = $c->request->params->{limit}    || $conf->{CNF_LIMIT()}  || undef;
    $page   = $c->request->params->{page}     || 1;
    $skip   = $c->model(HAWAT_MODEL_PAGINATOR())->calculate_skip($limit, $page) if $limit;

    eval {
        # Attempt to find the items in database
        my ($items, $count, $count_all, $duration, $query, @more) = $model->$operation($dbquery, $limit, $skip, $sortby, $sort, $quick, @args);

        # Put the results to the stash
        $c->stash($conf->{CNF_ITEMS_S()}              => $items,
                  $conf->{CNF_ITEMS_S()}.'_count'     => $count,
                  $conf->{CNF_ITEMS_S()}.'_count_all' => $count_all,
                  $conf->{CNF_ITEMS_S()}.'_duration'  => $duration,
        );
        if (@more) { $c->stash($conf->{CNF_ITEMS_S()}.'_more' => \@more); }

        # In case of the limit was given, calculate the paginator data
        if ($limit) {
            $c->stash($conf->{CNF_ITEMS_S()}.'_paginator' => $c->model(HAWAT_MODEL_PAGINATOR())->calculate($limit, $page, $count, $count_all));
            $c->stash->{$conf->{CNF_ITEMS_S()}.'_paginator'}->{type} = ($quick)?'simple':'advanced';
        }

        # TODO: make use of 'nowrap' for ajax queries (see the site/wrapper template
        # for explanation, why this works)
        #$c->stash(nowrap => 1);

        # Optionally store the database query - dependent on the DEVEL switch
        $self->store_query($c, 'Find '.$conf->{CNF_ITEMS_S()}, $query);

        # Optionally dump interesting variables - dependent on the DEVEL switch
        $self->dump_variable($c, 'Find '.$conf->{CNF_ITEMS_S()}.' args', $dbquery);
        $self->dump_variable($c, 'Find '.$conf->{CNF_ITEMS_S()}, $items);
    };
    # Handle any database exceptions, but make sure never to catch 'detach' event
    # from catalyst internals
    if ($@) {
        die $@ if $@ =~ /catalyst_detach/;
        my $message = 'Server was unable to process your search request. Please try again better.';
        $c->detach(HAWAT_ERROR_HANDLER, [message => $message, message_dev => "$@", menu => $conf->{CNF_MENU()}]);
        return; # Only for readability, detach works through DIE
    }
}

=head2 list CONF [ARG, ...]

List items in database according to the given rules

B<Arguments:>

=over

=item I<CONF> [HASH REFERENCE, MANDATORY]

Configuration hash, which controls inner behaviour of the action. This action accepts
following configurations (please see section L</"ACTION CONFIGURATIONS"> for descriptions):

C<items[O], limit[O], menu[O], model[M], search[O], sortby[O], sort[O]>

Note: [M] = mandatory configuration, [O] = optional configuration

=item I<ARGS> [MIXED, OPTIONAL]

Optional additional arguments, will be passed directly to the models I<find()> method

=back

=cut

sub list :Private {
    my ($self, $c, $conf, @args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Perform configuration check and normalization
    $self->_chcnf($c, $conf, CNF_MODEL);

    # Check, that the model can perform the requested list operation
    my ($model, $operation) = $self->_choper($c, HAWAT_ERROR_HANDLER, $conf);

    # Setup the search arguments
    my ($search, $limit, $sortby, $sort, $page, $skip);
    $search = $c->request->params->{search} || $conf->{CNF_SEARCH()} || undef;
    $limit  = $c->request->params->{limit}  || $conf->{CNF_LIMIT()}  || undef;
    $sortby = $c->request->params->{sortby} || $conf->{CNF_SORTBY()} || undef;
    $sort   = $c->request->params->{sort}   || $conf->{CNF_SORT()}   || undef;
    $page   = $c->request->params->{page}   || 1;
    $skip   = $c->model(HAWAT_MODEL_PAGINATOR())->calculate_skip($limit, $page) if $limit;

    eval {
        # Attempt to find the items in database
        my ($items, $count, $count_all, $duration, $query, @more) = $model->$operation($search, $limit, $skip, $sortby, $sort, @args);

        # Put the results to the stash
        $c->stash($conf->{CNF_ITEMS_S()}              => $items,
                  $conf->{CNF_ITEMS_S()}.'_count'     => $count,
                  $conf->{CNF_ITEMS_S()}.'_count_all' => $count_all,
                  $conf->{CNF_ITEMS_S()}.'_duration'  => $duration,
                  $conf->{CNF_ITEMS_S()}.'_search'    => $search,
        );
        if (@more) { $c->stash($conf->{CNF_ITEMS_S()}.'_more' => \@more); }

        # In case of the limit was given, calculate the paginator data
        if ($limit) {
            $c->stash($conf->{CNF_ITEMS_S()}.'_paginator' => $c->model(HAWAT_MODEL_PAGINATOR())->calculate($limit, $page, $count, $count_all));
        }

        # TODO: make use of 'nowrap' for ajax queries (see the site/wrapper template
        # for explanation, why this works)
        #$c->stash(nowrap => 1);

        # Optionally store the database query - dependent on the DEVEL switch
        $self->store_query($c, 'List '.$conf->{CNF_ITEMS()}, $query);

        # Optionally dump interesting variables - dependent on the DEVEL switch
        $self->dump_variable($c, 'List '.$conf->{CNF_ITEMS()}, $items);
    };
    # Handle any database exceptions, but make sure never to catch 'detach' event
    # from catalyst internals
    if ($@) {
        die $@ if $@ =~ /catalyst_detach/;
        $c->detach(HAWAT_ERROR_HANDLER, [message => "$@", menu => $conf->{CNF_MENU()}]);
        return; # Only for readability, detach works through DIE
    }
}

=head2 fetch CONF ID [ARG, ...]

Fetch the item from database

B<Arguments:>

=over

=item I<CONF> [HASH REFERENCE, MANDATORY]

Configuration hash, which controls inner behaviour of the action. This action accepts
following configurations (please see section L</"ACTION CONFIGURATIONS"> for descriptions):

C<item[O], menu[O], model[M], regexp_id[O]>

Note: [M] = mandatory configuration, [O] = optional configuration

=item I<ID> [STRING, MANDATORY]

Identifier of the item to be deleted

=item I<ARGS> [MIXED, OPTIONAL]

Optional additional arguments, will be passed directly to the models delete() method

=back

=cut

sub fetch :Private {
    my ($self, $c, $conf, $id, @args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Perform configuration check and normalization
    $self->_chcnf($c, $conf, CNF_MODEL);

    # Perform argument sanity check
    $self->_chid($c, HAWAT_ERROR_HANDLER, CNF_ITEM, $conf, $id);

    eval {
        # Attempt to fetch the item from database
        my ($item, $count, $duration, @more) = $c->model($conf->{CNF_MODEL()})->fetch($id, @args);

        # Put the results to the stash
        $c->stash($conf->{CNF_ITEM_S()}."_id"       => $id,
                  $conf->{CNF_ITEM_S()}             => $item,
                  $conf->{CNF_ITEM_S()}."_count"    => $count,
                  $conf->{CNF_ITEM_S()}."_duration" => $duration,
        );
        if (@more) { $c->stash($conf->{CNF_ITEM_S()}."_more" => \@more); }

        # Optionally dump interesting variables - dependent on the DEVEL switch
        $self->dump_variable($c, 'Fetch '.ucfirst($conf->{CNF_ITEM()}), $item);
    };
    # Handle any database exceptions, but make sure never to catch 'detach' event
    # from catalyst internals
    if ($@) {
        die $@ if $@ =~ /catalyst_detach/;
        $c->detach(HAWAT_ERROR_HANDLER, [message => "$@", menu => $conf->{CNF_MENU()}]);
        return; # Only for readability, detach works through DIE
    }
}

=head2 create CONF [ARG, ...]

Create item in database according to the given rules

B<Arguments:>

=over

=item I<CONF> [HASH REFERENCE, MANDATORY]

Configuration hash, which controls inner behaviour of the action. This action accepts
following configurations (please see section L</"ACTION CONFIGURATIONS"> for descriptions):

C<controller[M], model[M], item[O], menu[O]>

Note: [M] = mandatory configuration, [O] = optional configuration

=item I<ARGS> [MIXED, OPTIONAL]

Optional additional arguments, will be passed directly to the models delete() method

=back

=cut

sub create :Private {
    my ($self, $c, $conf, @args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Perform configuration check and normalization
    $self->_chcnf($c, $conf, CNF_CONTROLLER, CNF_MODEL, CNF_BACKLINK);

    # Set the type of the operation
    $c->stash($conf->{CNF_ITEM_S()}.'_form_action' => 'create');

    eval {
        # Fetch the form data (submitted using POST method)
        my $form = $c->request->body_params;

        # If the form was correctly submitted, attempt to insert the item to the database
        if ($form->{submitted}) {
            # Optionally dump interesting variables - dependent on the DEVEL switch
            $self->dump_variable($c, 'Create '.ucfirst($conf->{CNF_ITEM()}).' form', $form, 'session');

            # Check for cancelation button
            if ($form->{submit} eq HAWAT_FORM_CANCEL()) {
                $self->flash_info($c, ucfirst($conf->{CNF_ITEM()})." creation canceled.");
                $c->response->redirect($conf->{CNF_BACKLINK()});
                return;
            }

            my ($item_object, $errors) = $c->model($conf->{CNF_MODEL()})->create_object($form, @args);
            if ($errors and @$errors) {
                $c->stash($conf->{CNF_ITEM_S()}.'_form_errors' => $errors);
            }
            else {
                # Detect and filter out the preview action
                unless ($form->{submit} eq HAWAT_FORM_PREVIEW()) {
                    # Attempt to insert the item into the database
                    my ($id, $errors) = $c->model($conf->{CNF_MODEL()})->create($item_object, @args);

                    # SUCCESS, redirect to detail page in appropriate controller
                    if ($id) {
                        my $msg_s = ucfirst($conf->{CNF_ITEM()})." '$id' was successfully created in database.";
                        $self->log_change($c, 'create', $conf->{CNF_ITEM()}, $id, $msg_s);
                        $self->flash_success($c, $msg_s);
                        $c->response->redirect($c->uri_for($c->controller($conf->{CNF_CONTROLLER()})->action_for('view'),$id));
                        return;
                    }
                    # FAILURE, stash the errors to be displayed
                    else {
                        $c->stash($conf->{CNF_ITEM_S()}.'_form_errors' => $errors);
                    }
                }
            }

            # Put the object to the stash
            $c->stash($conf->{CNF_ITEM_S()}.'_form' => $item_object);
        }

        # TODO: make use of 'nowrap' for ajax queries (see the site/wrapper template
        # for explanation, why this works)
        #$c->stash(nowrap => 1);
    };
    # Handle any database exceptions, but make sure never to catch 'detach' event
    # from catalyst internals
    if ($@) {
        die $@ if $@ =~ /catalyst_detach/;
        $c->detach(HAWAT_ERROR_HANDLER, [message => "$@", menu => $conf->{CNF_MENU()}]);
        return; # Only for readability, detach works through DIE
    }
}

=head2 update CONF ID [ARG, ...]

Update item in database according to the given rules

B<Arguments:>

=over

=item I<CONF> [HASH REFERENCE, MANDATORY]

Configuration hash, which controls inner behaviour of the action. This action accepts
following configurations (please see section L</"ACTION CONFIGURATIONS"> for descriptions):

C<model[M], item[O], menu[O], backlink[M], regexp_id[O], modification[0]>

Note: [M] = mandatory configuration, [O] = optional configuration

=item I<ARGS> [MIXED, OPTIONAL]

Optional additional arguments, will be passed directly to the models delete() method

=back

=cut

sub update :Private {
    my ($self, $c, $conf, $id, @args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Perform configuration check and normalization
    $self->_chcnf($c, $conf, CNF_MODEL, CNF_BACKLINK);

    # Check, that the model can perform the requested modification operation
    my ($model, $modification) = $self->_choper($c, HAWAT_ERROR_HANDLER, $conf);

    # Perform argument sanity check
    $self->_chid($c, HAWAT_ERROR_HANDLER, CNF_ITEM, $conf, $id);

    # Attempt to fetch the item from database
    my $item = $self->_chitem($c, HAWAT_ERROR_HANDLER, $conf, $id, 'update', @args);
    # Put the item to the stash
    $c->stash($conf->{CNF_ITEM_S()}.'_form' => $item);
    # Set the type of the operation
    $c->stash($conf->{CNF_ITEM_S()}.'_form_action' => $modification);

    eval {
        # Fetch the form data (submitted using POST method)
        my $form = $c->request->body_params;

        # If the form was correctly submitted, attempt to insert the item to the database
        if ($form->{submitted}) {
            # Optionally dump interesting variables - dependent on the DEVEL switch
            $self->dump_variable($c, ucfirst($modification).' '.ucfirst($conf->{CNF_ITEM()}).' form', $form, 'session');

            # Check for cancelation button
            if ($form->{submit} eq HAWAT_FORM_CANCEL()) {
                $self->flash_info($c, ucfirst($conf->{CNF_ITEM()})." update canceled.");
                $c->response->redirect($conf->{CNF_BACKLINK()});
                return;
            }

            # Merge the item object with changes from form
            my $mo = $modification."_object";
            my ($item_object, $errors) = $model->$mo($item, $form, @args);
            if ($errors and @$errors) {
                $c->stash($conf->{CNF_ITEM_S()}.'_form_errors' => $errors);
            }
            else {
                # Optionally dump interesting variables - dependent on the DEVEL switch
                $self->dump_variable($c, ucfirst($modification).' '.$conf->{CNF_ITEM()}.' after merge', $item_object, 'session');

                # Detect and filter out the preview action
                unless ($form->{submit} eq HAWAT_FORM_PREVIEW()) {
                    # Attempt to update the item within the database
                    my ($rv, $errors) = $model->$modification($id, $item_object, @args);

                    # Optionally dump interesting variables - dependent on the DEVEL switch
                    $self->dump_variable($c, ucfirst($modification).' '.$conf->{CNF_ITEM()}.' result', $rv, 'session');

                    # SUCCESS, redirect to previous page
                    if ($rv and $rv->{ok} and $rv->{n}) {
                        my $msg_s = ucfirst($conf->{CNF_ITEM()})." '$id' was successfully updated in database. [#$rv->{n}]";
                        $self->log_change($c, 'update', $conf->{CNF_ITEM()}, $id, $msg_s);
                        $self->flash_success($c, $msg_s);
                        $c->response->redirect($conf->{CNF_BACKLINK()});
                        return;
                    }
                    # DATABASE FAILURE, redirect to previous page and display error message
                    elsif ($rv) {
                        $self->flash_error($c, 'Unable to update '.$conf->{CNF_ITEM()}." '$id' in database.");
                        $c->response->redirect($conf->{CNF_BACKLINK()});
                        return;
                    }
                    # FORM FAILURE, stash the data and errors to be displayed
                    # NOTE: This should not happen anymore
                    else {
                        $c->stash($conf->{CNF_ITEM_S()}.'_form_errors' => $errors);
                    }
                }
            }

            # Put the object to the stash
            $c->stash($conf->{CNF_ITEM_S()}.'_form' => $item_object);
        }

        # TODO: make use of 'nowrap' for ajax queries (see the site/wrapper template
        # for explanation, why this works)
        #$c->stash(nowrap => 1);
    };
    # Handle any database exceptions, but make sure never to catch 'detach' event
    # from catalyst internals
    if ($@) {
        die $@ if $@ =~ /catalyst_detach/;
        $c->detach(HAWAT_ERROR_HANDLER, [message => "$@", menu => $conf->{CNF_MENU()}]);
        return; # Only for readability, detach works through DIE
    }
}

=head2 modify CONF ID [ARG, ...]

Modify the item in database

B<Arguments:>

=over

=item I<CONF> [HASH REFERENCE, MANDATORY]

Configuration hash, which controls inner behaviour of the action. This action accepts
following configurations (please see section L</"ACTION CONFIGURATIONS"> for descriptions):

C<controller[M], model[M], modification[M], item[M], menu[O], backlink[M], multi[O], msg_success[O], msg_none[O], msg_failure[O]>

Note: [M] = mandatory configuration, [O] = optional configuration

=item I<ID> [STRING, MANDATORY]

Identifier of the item to be modified

=item I<ARGS> [MIXED, OPTIONAL]

Optional additional arguments, will be passed directly to the models delete() method

=back

=cut

sub modify :Private {
    my ($self, $c, $conf, $id, @args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Perform configuration check and normalization
    $self->_chcnf($c, $conf, CNF_CONTROLLER, CNF_MODEL, CNF_OPERATION, CNF_BACKLINK);

    # Check, that the model can perform the requested modification operation
    my ($model, $modification) = $self->_choper($c, HAWAT_ERROR_HANDLER, $conf);

    # Perform argument sanity check
    $self->_chid($c, HAWAT_ERROR_HANDLER_R, CNF_ITEM, $conf, $id);

    eval {
        # Attempt to modify the item in database
        my $rv = $model->$modification($id, @args);
        #my $rv = { 'ok' => '1', 'n' => 1, 'connectionId' => 270, 'err' => undef };
        #SUCCESS: { 'ok' => '1', 'n' => 1, 'connectionId' => 270, 'err' => undef }
        #FAILURE: { 'ok' => '1', 'n' => 0, 'connectionId' => 270, 'err' => undef }

        # Optionally dump interesting variables - dependent on the DEVEL switch
        $self->dump_variable($c, ucfirst($modification).' '.$conf->{CNF_ITEM()}.' result', $rv, 'session');

        my $message;
        if ($rv->{ok} and $rv->{n} > 0) {
            $message = $conf->{CNF_MSG_SUCCESS()} || ucfirst($conf->{CNF_ITEM()})." '$id' was successfully modified in database.";
            $message .= " [#$rv->{n}]";
            $self->log_change($c, $modification, $conf->{CNF_ITEM()}, $id, $message);
            $self->flash_success($c, $message);
            $c->response->redirect($conf->{CNF_BACKLINK()}); return;
        }
        elsif ($rv->{ok} and $conf->{multi}) {
            $message = $conf->{msg_none} || "No ".ucfirst($conf->{CNF_ITEM()})." was modified in database.";
            $self->flash_info($c, $message);
            $c->response->redirect($conf->{CNF_BACKLINK()}); return;
        }
        else {
            $message = $conf->{CNF_MSG_FAILURE()} || "Unable to modify ".$conf->{CNF_ITEM()}." '$id' in database.";
            $c->detach(HAWAT_ERROR_HANDLER_R, [message => $message, backlink => $conf->{CNF_BACKLINK()}]);
            return; # Only for readability, detach works through DIE
        }
    };
    # Handle any database exceptions, but make sure never to catch 'detach' event
    # from catalyst internals
    if ($@) {
        die $@ if $@ =~ /catalyst_detach/;
        $c->detach(HAWAT_ERROR_HANDLER_R, [message => "$@", backlink => $conf->{CNF_BACKLINK()}]);
        return; # Only for readability, detach works through DIE
    }
}

=head2 rename CONF ID [ARG, ...]

Rename item in database according to the given rules (change item ID)

B<Arguments:>

=over

=item I<CONF> [HASH REFERENCE, MANDATORY]

Configuration hash, which controls inner behaviour of the action. This action accepts
following configurations (please see section L</"ACTION CONFIGURATIONS"> for descriptions):

C<model[M], item[O], menu[O], backlink[M], regexp_id[O]>

Note: [M] = mandatory configuration, [O] = optional configuration

=item I<ARGS> [MIXED, OPTIONAL]

Optional additional arguments, will be passed directly to the models delete() method

=back

=cut

sub rename :Private {
    my ($self, $c, $conf, $id, @args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Perform configuration check and normalization
    $self->_chcnf($c, $conf, CNF_CONTROLLER, CNF_MODEL, CNF_BACKLINK);

    # Perform argument sanity check
    $self->_chid($c, HAWAT_ERROR_HANDLER, CNF_ITEM, $conf, $id);

    # Attempt to fetch the item from database
    my $item = $self->_chitem($c, HAWAT_ERROR_HANDLER, $conf, $id, 'rename', @args);
    # Put the item to the stash
    $c->stash('rename_form' => $item);

    eval {
        # Fetch the form data (submitted using POST method)
        my $form = $c->request->body_params;

        # If the form was correctly submitted, attempt to insert the item to the database
        if ($form->{submitted}) {
            # Optionally dump interesting variables - dependent on the DEVEL switch
            $self->dump_variable($c, 'Rename '.ucfirst($conf->{CNF_ITEM()}).' form', $form, 'session');

            # Check for cancelation button
            if ($form->{submit} eq HAWAT_FORM_CANCEL()) {
                $self->flash_info($c, ucfirst($conf->{CNF_ITEM()})." renaming canceled.");
                $c->response->redirect($conf->{CNF_BACKLINK()});
                return;
            }

            # Attempt to rename the item within the database
            my ($rv, $errors) = $c->model($conf->{CNF_MODEL()})->rename($id, $item, $form->{id}, @args, $c);

            # Optionally dump interesting variables - dependent on the DEVEL switch
            $self->dump_variable($c, 'Rename '.$conf->{CNF_ITEM()}.' result', $rv, 'session');

            # SUCCESS, redirect to previous page
            if ($rv) {
                my $msg_s = ucfirst($conf->{CNF_ITEM()})." '$id' was successfully renamed to '$rv' in database.";
                $self->log_change($c, 'rename', $conf->{CNF_ITEM()}, $id, $msg_s);
                $self->flash_success($c, $msg_s);
                $c->response->redirect($c->uri_for($c->controller($conf->{CNF_CONTROLLER()})->action_for('view'),$rv));
                return;
            }
            # FAILURE, stash the data and errors to be displayed
            else {
                $c->stash('rename_form_errors' => $errors);
                # NOTE: $item after update now contains merged object
                $c->stash('rename_form' => $item);
            }
        }

        # Set the type of the operation
        $c->stash($conf->{CNF_ITEM_S()}.'_form_action' => 'rename');

        # TODO: make use of 'nowrap' for ajax queries (see the site/wrapper template
        # for explanation, why this works)
        #$c->stash(nowrap => 1);
    };
    # Handle any database exceptions, but make sure never to catch 'detach' event
    # from catalyst internals
    if ($@) {
        die $@ if $@ =~ /catalyst_detach/;
        $c->detach(HAWAT_ERROR_HANDLER, [message => "$@", menu => $conf->{CNF_MENU()}]);
        return; # Only for readability, detach works through DIE
    }
}

=head2 delete CONF ID [ARG, ...]

Delete the item from database

B<Arguments:>

=over

=item I<CONF> [HASH REFERENCE, MANDATORY]

Configuration hash, which controls inner behaviour of the action. This action accepts
following configurations (please see section L</"ACTION CONFIGURATIONS"> for descriptions):

C<controller[M], model[M], item[M], menu[O], backlink[M], backlink_d[M]>

Note: [M] = mandatory configuration, [O] = optional configuration

=item I<ID> [STRING, MANDATORY]

Identifier of the item to be deleted

=item I<ARGS> [MIXED, OPTIONAL]

Optional additional arguments, will be passed directly to the models delete() method

=back

=cut

sub delete :Private {
    my ($self, $c, $conf, $id, $confirm, @args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Perform configuration check and normalization
    $self->_chcnf($c, $conf, CNF_CONTROLLER, CNF_MODEL, CNF_BACKLINK, CNF_BACKLINK_D);

    # Perform argument sanity check
    $self->_chid($c, HAWAT_ERROR_HANDLER_R, CNF_ITEM, $conf, $id);

    # Handle operation confirmation - display the question page
    unless ($confirm) {
        $c->stash(id => $id,
                  template => 'inc/dialog_confirmation.html.tt',
                  title => 'Hawat - '.ucfirst($conf->{CNF_ITEM()}).' deletion',
                  menu => $conf->{CNF_MENU()},
                  controller => $conf->{CNF_CONTROLLER()},
                  action_confirm => 'delete',
                  action_cancel => 'delete',
                  question => "Are you really sure you want to permanently delete ".$conf->{CNF_ITEM()}." '$id'?",
                  backlink => $conf->{CNF_BACKLINK()});
        # Render the view
        # NOTE: No need for that now, it will be rendered automagically, when the
        # 'end' method is called (see the Hawat::Controller::Root for details)
        $c->detach($c->view(HAWAT_VIEW_HTML));
        return; # Only for readability, detach works through DIE
    }
    # Handle the operation cancelation
    elsif (lc($confirm) =~ 'cancel|no') {
        $self->flash_info($c, "Deletion of ".$conf->{CNF_ITEM()}." '$id' was canceled.");
        $c->response->redirect($conf->{CNF_BACKLINK()});
        $c->detach;
        return; # Only for readability, detach works through DIE
    }

    eval {
        # Attempt to delete the item from database
        my $rv = $c->model($conf->{CNF_MODEL()})->delete($id, @args);
        #my $rv = { 'ok' => '1', 'n' => 1, 'connectionId' => 270, 'err' => undef };
        #SUCCESS: { 'ok' => '1', 'n' => 1, 'connectionId' => 270, 'err' => undef }
        #FAILURE: { 'ok' => '1', 'n' => 0, 'connectionId' => 270, 'err' => undef }

        # Optionally dump interesting variables - dependent on the DEVEL switch
        $self->dump_variable($c, 'Delete '.$conf->{CNF_ITEM()}.' result', $rv, 'session');

        my $message;
        if ($rv->{ok} and $rv->{n} == 1) {
            $message = $conf->{CNF_MSG_SUCCESS()} || ucfirst($conf->{CNF_ITEM()})." '$id' was successfully deleted from database.";
            $self->log_change($c, 'delete', $conf->{CNF_ITEM()}, $id, $message);
            $self->flash_success($c, $message);
            $c->response->redirect($conf->{CNF_BACKLINK_D()});
            return;
        }
        else {
            $message = $conf->{CNF_MSG_FAILURE()} || "Unable to delete ".$conf->{CNF_ITEM()}." '$id' from database.";
            $c->detach(HAWAT_ERROR_HANDLER_R, [message => $message, backlink => $conf->{CNF_BACKLINK()}]);
            return; # Only for readability, detach works through DIE
        }
    };
    # Handle any database exceptions, but make sure never to catch 'detach' event
    # from catalyst internals
    # Handle any exceptions, make sure never to catch 'detach' event from catalyst
    if ($@) {
        die $@ if $@ =~ /catalyst_detach/;
        $c->detach(HAWAT_ERROR_HANDLER_R, [message => "$@", backlink => $conf->{CNF_BACKLINK()}]);
        return; # Only for readability, detach works through DIE
    }
}

=head2 remove CONF [ARG, ...]

Delete the item from database

B<Arguments:>

=over

=item I<CONF> [HASH REFERENCE, MANDATORY]

Configuration hash, which controls inner behaviour of the action. This action accepts
following configurations (please see section L</"ACTION CONFIGURATIONS"> for descriptions):

C<controller[M], model[M], item[M], menu[O], backlink[M], backlink_d[M]>

Note: [M] = mandatory configuration, [O] = optional configuration

=item I<ID> [STRING, MANDATORY]

Identifier of the item to be removed

=item I<ARGS> [MIXED, OPTIONAL]

Optional additional arguments, will be passed directly to the models remove() method

=back

=cut

sub remove :Private {
    my ($self, $c, $conf, $confirm, @args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Perform configuration check and normalization
    $self->_chcnf($c, $conf, CNF_CONTROLLER, CNF_MODEL, CNF_BACKLINK, CNF_BACKLINK_D);

    # Fetch the
    my $filter = $c->request->params->{filter} || $conf->{CNF_FILTER()} || undef;

    # Handle operation confirmation - display the question page
    unless ($confirm) {
        $c->stash(template => 'inc/dialog_confirmation.html.tt',
                  title => 'Hawat - '.ucfirst($conf->{CNF_ITEM()}).' remove',
                  menu => $conf->{CNF_MENU()},
                  controller => $conf->{CNF_CONTROLLER()},
                  action_confirm => 'remove',
                  action_cancel => 'remove',
                  params => $c->request->params,
                  question => "Are you really sure you want to permanently remove all matching ".$conf->{CNF_ITEMS()}."?",
                  backlink => $conf->{CNF_BACKLINK()});
        # Render the view
        # NOTE: No need for that now, it will be rendered automagically, when the
        # 'end' method is called (see the Hawat::Controller::Root for details)
        $c->detach($c->view(HAWAT_VIEW_HTML));
        return; # Only for readability, detach works through DIE
    }
    # Handle the operation cancelation
    elsif (lc($confirm) =~ 'cancel|no') {
        $self->flash_info($c, "Removal of ".$conf->{CNF_ITEMS()}." was canceled.");
        $c->response->redirect($conf->{CNF_BACKLINK()});
        $c->detach;
        return; # Only for readability, detach works through DIE
    }

    eval {
        # Attempt to remove the item from database
        my $rv = $c->model($conf->{CNF_MODEL()})->remove($filter, @args);
        #my $rv = { 'ok' => '1', 'n' => 1, 'connectionId' => 270, 'err' => undef };
        #SUCCESS: { 'ok' => '1', 'n' => 1, 'connectionId' => 270, 'err' => undef }
        #FAILURE: { 'ok' => '1', 'n' => 0, 'connectionId' => 270, 'err' => undef }

        # Optionally dump interesting variables - dependent on the DEVEL switch
        $self->dump_variable($c, 'Remove '.$conf->{CNF_ITEM()}.' result', $rv, 'session');

        my $message;
        if ($rv->{ok} and $rv->{n} > 0) {
            $message = $conf->{CNF_MSG_SUCCESS()} || ucfirst($conf->{CNF_ITEMS()})." were successfully removed from database. (#".$rv->{n}.")";
            $self->log_change($c, 'remove', $conf->{CNF_ITEMS()}, '', $message);
            $self->flash_success($c, $message);
            $c->response->redirect($conf->{CNF_BACKLINK_D()});
            return;
        }
        else {
            $message = $conf->{CNF_MSG_FAILURE()} || "Unable to remove ".$conf->{CNF_ITEMS()}." from database.";
            $c->detach(HAWAT_ERROR_HANDLER_R, [message => $message, backlink => $conf->{CNF_BACKLINK()}]);
            return; # Only for readability, detach works through DIE
        }
    };
    # Handle any database exceptions, but make sure never to catch 'detach' event
    # from catalyst internals
    # Handle any exceptions, make sure never to catch 'detach' event from catalyst
    if ($@) {
        die $@ if $@ =~ /catalyst_detach/;
        $c->detach(HAWAT_ERROR_HANDLER_R, [message => "$@", backlink => $conf->{CNF_BACKLINK()}]);
        return; # Only for readability, detach works through DIE
    }
}

=head2 subCreate CONF [ARG, ...]

Create subitem in database according to the given rules

B<Arguments:>

=over

=item I<CONF> [HASH REFERENCE, MANDATORY]

Configuration hash, which controls inner behaviour of the action. This action accepts
following configurations (please see section L</"ACTION CONFIGURATIONS"> for descriptions):

C<controller[M], model[M], subitem[M], item[O], menu[O], modification[M]>

Note: [M] = mandatory configuration, [O] = optional configuration

=item I<ARGS> [MIXED, OPTIONAL]

Optional additional arguments, will be passed directly to the models delete() method

=back

=cut

sub subCreate :Private {
    my ($self, $c, $conf, $id, @args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Perform configuration check and normalization
    $self->_chcnf($c, $conf, CNF_CONTROLLER, CNF_MODEL, CNF_BACKLINK, CNF_OPERATION, CNF_SUBITEM);

    # Check, that the model can perform the requested modification operation
    my ($model, $modification) = $self->_choper($c, HAWAT_ERROR_HANDLER, $conf);

    # Perform argument sanity check
    $self->_chid($c, HAWAT_ERROR_HANDLER, CNF_ITEM, $conf, $id);

    # Attempt to fetch the item from database
    my $item = $self->_chitem($c, HAWAT_ERROR_HANDLER, $conf, $id, $modification, @args);

    eval {
        # Fetch the form data (submitted using POST method)
        my $form = $c->request->body_params;

        # If the form was correctly submitted, attempt to insert the item to the database
        if ($form->{submitted}) {
            # Optionally dump interesting variables - dependent on the DEVEL switch
            #$self->dump_variable($c, ucfirst($modification).' '.ucfirst($conf->{CNF_SUBITEM()}).' form', $form, 'session');

            # Check for cancelation button
            if ($form->{submit} eq HAWAT_FORM_CANCEL()) {
                $self->flash_info($c, ucfirst($conf->{CNF_SUBITEM()})." update canceled.");
                $c->response->redirect($conf->{CNF_BACKLINK()});
                return;
            }

            # Merge the item object with changes from form
            my $cso = 'create_'.$conf->{CNF_SUBITEM()};
            my ($subitem_object, $errors) = $model->$cso($form, @args);
            # NOTE: $item after update now contains merged object
            $c->stash($conf->{CNF_SUBITEM()}.'_form' => $subitem_object);
            if ($errors and @$errors) {
                $c->stash($conf->{CNF_SUBITEM()}.'_form_errors' => $errors);
            }
            else {
                # Optionally dump interesting variables - dependent on the DEVEL switch
                #$self->dump_variable($c, ucfirst($conf->{CNF_ITEM()}).' subitem object: ', $subitem_object, 'session');

                # Detect and filter out the preview and autofill actions
                unless ($form->{submit} eq HAWAT_FORM_PREVIEW() or $form->{submit} eq HAWAT_FORM_AUTOFILL()) {
                    # Optionally dump interesting variables - dependent on the DEVEL switch
                    #$self->dump_variable($c, ucfirst($conf->{CNF_ITEM()}).' before '.$modification, $item, 'session');

                    # Attempt to update the item within the database
                    my ($rv, $errors) = $model->$modification($id, $subitem_object, @args);

                    # Optionally dump interesting variables - dependent on the DEVEL switch
                    $self->dump_variable($c, ucfirst($modification).' '.$conf->{CNF_ITEM()}.' result', $rv, 'session');

                    # SUCCESS, redirect to previous page
                    if ($rv and $rv->{ok} and $rv->{n}) {
                        my $msg_s = ucfirst($conf->{CNF_SUBITEM()})." '$id' was successfully created in database. [#$rv->{n}]";
                        $self->log_change($c, 'subCreate', $conf->{CNF_SUBITEM()}, $id, $msg_s);
                        $self->flash_success($c, $msg_s);
                        $c->response->redirect($conf->{CNF_BACKLINK()});
                        return;
                    }
                    # DATABASE FAILURE, redirect to previous page and display error message
                    elsif ($rv) {
                        $self->flash_error($c, 'Unable to create new '.$conf->{CNF_SUBITEM()}." '$id' in database.");
                        $c->response->redirect($conf->{CNF_BACKLINK()});
                        return;
                    }
                    # FORM FAILURE, stash the data and errors to be displayed
                    else {
                        $c->stash($conf->{CNF_SUBITEM()}.'_form_errors' => $errors);
                    }
                }
            }
        }

        # Set the type of the operation
        $c->stash($conf->{CNF_SUBITEM()}.'_form_action' => $modification);

        # TODO: make use of 'nowrap' for ajax queries (see the site/wrapper template
        # for explanation, why this works)
        #$c->stash(nowrap => 1);
    };
    # Handle any database exceptions, but make sure never to catch 'detach' event
    # from catalyst internals
    if ($@) {
        die $@ if $@ =~ /catalyst_detach/;
        $c->detach(HAWAT_ERROR_HANDLER, [message => "$@", menu => $conf->{CNF_MENU()}]);
        return; # Only for readability, detach works through DIE
    }
}

=head2 subUpdate CONF ID [ARG, ...]

Update item in database according to the given rules

B<Arguments:>

=over

=item I<CONF> [HASH REFERENCE, MANDATORY]

Configuration hash, which controls inner behaviour of the action. This action accepts
following configurations (please see section L</"ACTION CONFIGURATIONS"> for descriptions):

C<model[M], item[O], menu[O], backlink[M], regexp_id[O], modification[0]>

Note: [M] = mandatory configuration, [O] = optional configuration

=item I<ARGS> [MIXED, OPTIONAL]

Optional additional arguments, will be passed directly to the models delete() method

=back

=cut

sub subUpdate :Private {
    my ($self, $c, $conf, $id, $sid, @args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Perform configuration check and normalization
    $self->_chcnf($c, $conf, CNF_MODEL, CNF_BACKLINK, CNF_OPERATION, CNF_SUBITEM);

    # Check, that the model can perform the requested modification operation
    my ($model, $modification) = $self->_choper($c, HAWAT_ERROR_HANDLER, $conf);

    # Perform argument sanity check
    $self->_chid($c, HAWAT_ERROR_HANDLER, CNF_ITEM, $conf, $id);
    $self->_chid($c, HAWAT_ERROR_HANDLER, CNF_SUBITEM, $conf, $sid);

    # Attempt to fetch the item from database
    my $item = $self->_chitem($c, HAWAT_ERROR_HANDLER, $conf, $id, $modification, @args);

    eval {
        my $subitem = $self->_fetchSubitem($c, $item, $conf, $sid);
        unless ($subitem) {
            $c->detach(HAWAT_ERROR_HANDLER, [message => ucfirst($conf->{CNF_SUBITEM()})." '$sid' was not found in database.", menu => $conf->{CNF_MENU()}]);
            return; # Only for readability, detach works through DIE
        }
        # Put the item to the stash
        $c->stash($conf->{CNF_SUBITEM()}.'_form' => $subitem);

        # Optionally dump interesting variables - dependent on the DEVEL switch
        #$self->dump_variable($c, ucfirst($conf->{CNF_ITEM()}).' before '.$modification, $item, 'session');
        #$self->dump_variable($c, ucfirst($conf->{CNF_SUBITEM()}).' before '.$modification, $subitem, 'session');

        # Fetch the form data (submitted using POST method)
        my $form = $c->request->body_params;

        # If the form was correctly submitted, attempt to insert the item to the database
        if ($form->{submitted}) {
            # Optionally dump interesting variables - dependent on the DEVEL switch
            #$self->dump_variable($c, ucfirst($modification).' '.ucfirst($conf->{CNF_SUBITEM()}).' form', $form, 'session');

            # Check for cancelation button
            if ($form->{submit} eq HAWAT_FORM_CANCEL()) {
                $self->flash_info($c, ucfirst($conf->{CNF_SUBITEM()})." update canceled.");
                $c->response->redirect($conf->{CNF_BACKLINK()});
                return;
            }

            # Merge the item object with changes from form
            my $cso = 'create_'.$conf->{CNF_SUBITEM()};
            my ($subitem_object, $errors) = $model->$cso($form, @args);
            # NOTE: $item after update now contains merged object
            $c->stash($conf->{CNF_SUBITEM()}.'_form' => $subitem_object);
            if ($errors and @$errors) {
                $c->stash($conf->{CNF_SUBITEM()}.'_form_errors' => $errors);
            }
            else {
                # Optionally dump interesting variables - dependent on the DEVEL switch
                #$self->dump_variable($c, ucfirst($conf->{CNF_ITEM()}).' subitem object: ', $subitem_object, 'session');

                # Detect and filter out the preview and autofill actions
                unless ($form->{submit} eq HAWAT_FORM_PREVIEW() or $form->{submit} eq HAWAT_FORM_AUTOFILL()) {
                    # Attempt to update the item within the database
                    my ($rv, $errors) = $model->$modification($id, $sid, $subitem_object, @args);

                    # Optionally dump interesting variables - dependent on the DEVEL switch
                    $self->dump_variable($c, 'Update '.$conf->{CNF_ITEM()}.' result', $rv, 'session');

                    # SUCCESS, redirect to previous page
                    if ($rv and $rv->{ok} and $rv->{n}) {
                        my $msg_s = ucfirst($conf->{CNF_SUBITEM()})." '$id' was successfully updated in database. [#$rv->{n}]";
                        $self->log_change($c, 'subUpdate', $conf->{CNF_SUBITEM()}, $id, $msg_s);
                        $self->flash_success($c, $msg_s);
                        $c->response->redirect($conf->{CNF_BACKLINK()});
                        return;
                    }
                    # DATABASE FAILURE, redirect to previous page and display error message
                    elsif ($rv) {
                        $self->flash_error($c, 'Unable to update '.$conf->{CNF_SUBITEM()}." '$id' in database.");
                        $c->response->redirect($conf->{CNF_BACKLINK()});
                        return;
                    }
                    # FORM FAILURE, stash the data and errors to be displayed
                    else {
                        $c->stash($conf->{CNF_SUBITEM()}.'_form_errors' => $errors);
                    }
                }
            }
        }

        # Set the type of the operation
        $c->stash($conf->{CNF_SUBITEM()}.'_form_action' => $modification);

        # TODO: make use of 'nowrap' for ajax queries (see the site/wrapper template
        # for explanation, why this works)
        #$c->stash(nowrap => 1);
    };
    # Handle any database exceptions, but make sure never to catch 'detach' event
    # from catalyst internals
    if ($@) {
        die $@ if $@ =~ /catalyst_detach/;
        $c->detach(HAWAT_ERROR_HANDLER, [message => "$@", menu => $conf->{CNF_MENU()}]);
        return; # Only for readability, detach works through DIE
    }
}

=head2 subDelete CONF ID [ARG, ...]

Update item in database according to the given rules

B<Arguments:>

=over

=item I<CONF> [HASH REFERENCE, MANDATORY]

Configuration hash, which controls inner behaviour of the action. This action accepts
following configurations (please see section L</"ACTION CONFIGURATIONS"> for descriptions):

C<model[M], item[O], menu[O], backlink[M], regexp_id[O], modification[0]>

Note: [M] = mandatory configuration, [O] = optional configuration

=item I<ARGS> [MIXED, OPTIONAL]

Optional additional arguments, will be passed directly to the models delete() method

=back

=cut

sub subDelete :Private {
    my ($self, $c, $conf, $id, $sid, $confirm, @args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Perform configuration check and normalization
    $self->_chcnf($c, $conf, CNF_CONTROLLER, CNF_MODEL, CNF_ACTION, CNF_BACKLINK, CNF_OPERATION, CNF_SUBITEM);

    # Check, that the model can perform the requested modification operation
    my ($model, $modification) = $self->_choper($c, HAWAT_ERROR_HANDLER, $conf);

    # Perform argument sanity check
    $self->_chid($c, HAWAT_ERROR_HANDLER, CNF_ITEM, $conf, $id);
    $self->_chid($c, HAWAT_ERROR_HANDLER, CNF_SUBITEM, $conf, $sid);

    # Handle operation confirmation - display the question page
    unless ($confirm) {
        $c->stash(id => $id,
                  sid => $sid,
                  template => 'inc/dialog_confirmation.html.tt',
                  title => 'Hawat - '.ucfirst($conf->{CNF_SUBITEM()}).' deletion',
                  menu => $conf->{CNF_MENU()},
                  controller => $conf->{CNF_CONTROLLER()},
                  action_confirm => $conf->{CNF_ACTION()},
                  action_cancel => $conf->{CNF_ACTION()},
                  question => "Are you really sure you want to permanently delete ".$conf->{CNF_SUBITEM()}." '$sid' from ".$conf->{CNF_ITEM()}." '$id'?",
                  backlink => $conf->{CNF_BACKLINK()});
        # Render the view
        # NOTE: No need for that now, it will be rendered automagically, when the
        # 'end' method is called (see the Hawat::Controller::Root for details)
        $c->detach($c->view(HAWAT_VIEW_HTML));
        return; # Only for readability, detach works through DIE
    }
    # Handle the operation cancelation
    elsif (lc($confirm) =~ 'cancel|no') {
        $self->flash_info($c, "Deletion of ".$conf->{CNF_SUBITEM()}." '$sid' from ".$conf->{CNF_ITEM()}." '$id' was canceled.");
        $c->response->redirect($conf->{CNF_BACKLINK()});
        $c->detach;
        return; # Only for readability, detach works through DIE
    }

    eval {
        # Attempt to delete the item from database
        my $rv = $model->$modification($id, $sid, @args);
        #my $rv = { 'ok' => '1', 'n' => 1, 'connectionId' => 270, 'err' => undef };
        #SUCCESS: { 'ok' => '1', 'n' => 1, 'connectionId' => 270, 'err' => undef }
        #FAILURE: { 'ok' => '1', 'n' => 0, 'connectionId' => 270, 'err' => undef }

        # Optionally dump interesting variables - dependent on the DEVEL switch
        $self->dump_variable($c, 'Delete '.$conf->{CNF_SUBITEM()}.' result', $rv, 'session');

        my $message;
        if ($rv->{ok} and $rv->{n} == 1) {
            $message = $conf->{CNF_MSG_SUCCESS()} || ucfirst($conf->{CNF_SUBITEM()})." '$sid' was successfully deleted from ".$conf->{CNF_ITEM()}." '$id'.";
            $self->log_change($c, 'subDelete', $conf->{CNF_SUBITEM()}, $id, $message);
            $self->flash_success($c, $message);
            $c->response->redirect($conf->{CNF_BACKLINK()});
            return;
        }
        else {
            $message = $conf->{CNF_MSG_FAILURE()} || "Unable to delete ".$conf->{CNF_SUBITEM()}." '$sid' from ".$conf->{CNF_ITEM()}." '$id'.";
            $c->detach(HAWAT_ERROR_HANDLER_R, [message => $message, backlink => $conf->{CNF_BACKLINK()}]);
            return; # Only for readability, detach works through DIE
        }
    };
    # Handle any database exceptions, but make sure never to catch 'detach' event
    # from catalyst internals
    # Handle any exceptions, make sure never to catch 'detach' event from catalyst
    if ($@) {
        die $@ if $@ =~ /catalyst_detach/;
        $c->detach(HAWAT_ERROR_HANDLER_R, [message => "$@", backlink => $conf->{CNF_BACKLINK()}]);
        return; # Only for readability, detach works through DIE
    }
}

=head2 do CONF [ARG, ...]

Perform generic database operation

B<Arguments:>

=over

=item I<CONF> [HASH REFERENCE, MANDATORY]

Configuration hash, which controls inner behaviour of the action. This action accepts
following configurations (please see section L</"ACTION CONFIGURATIONS"> for descriptions):

C<controller[M], model[M], modification[M], item[M], menu[O], backlink[M], multi[O], msg_success[O], msg_none[O], msg_failure[O]>

Note: [M] = mandatory configuration, [O] = optional configuration

=item I<ID> [STRING, MANDATORY]

Identifier of the item to be modified

=item I<ARGS> [MIXED, OPTIONAL]

Optional additional arguments, will be passed directly to the models delete() method

=back

=cut

sub do :Private {
    my ($self, $c, $conf, @args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);

    # Perform configuration check and normalization
    $self->_chcnf($c, $conf, CNF_MODEL, CNF_BACKLINK);

    # Check, that the model can perform the requested modification operation
    my ($model, $modification) = $self->_choper($c, HAWAT_ERROR_HANDLER, $conf);

    # Set the type of the operation
    $c->stash($conf->{CNF_ITEM_S()}.'_form_action' => $modification);

    eval {
        # Fetch the form data (submitted using POST method)
        my $form = $c->request->body_params;

        # If the form was correctly submitted, attempt to insert the item to the database
        if ($form->{submitted}) {
            # Optionally dump interesting variables - dependent on the DEVEL switch
            $self->dump_variable($c, ucfirst($modification).' '.ucfirst($conf->{CNF_ITEM()}).' form', $form, 'session');

            # Check for cancelation button
            if ($form->{submit} eq HAWAT_FORM_CANCEL()) {
                $self->flash_info($c, ucfirst($conf->{CNF_ITEM()})." $modification canceled.");
                $c->response->redirect($conf->{CNF_BACKLINK()});
                return;
            }

            my ($result, $errors) = $model->$modification($form, @args);
            if ($errors and @$errors) {
                $c->stash($conf->{CNF_ITEM_S()}.'_form_errors' => $errors);
            }
            else {
                # Optionally dump interesting variables - dependent on the DEVEL switch
                $self->dump_variable($c, ucfirst($modification).' '.$conf->{CNF_ITEM()}.' result', $result, 'session');

                foreach my $r (@$result) {
                    if ($r->[0] eq 'success') { $self->flash_success($c, $r->[1]); }
                    else                      { $self->flash_error($c, $r->[1]); }
                }
                $c->response->redirect($conf->{CNF_BACKLINK()});
                return;
            }

            # Put the object to the stash
            $c->stash($conf->{CNF_ITEM_S()}.'_form' => $form);
        }

        # TODO: make use of 'nowrap' for ajax queries (see the site/wrapper template
        # for explanation, why this works)
        #$c->stash(nowrap => 1);
    };
    # Handle any database exceptions, but make sure never to catch 'detach' event
    # from catalyst internals
    if ($@) {
        die $@ if $@ =~ /catalyst_detach/;
        $c->detach(HAWAT_ERROR_HANDLER, [message => "$@", menu => $conf->{CNF_MENU()}]);
        return; # Only for readability, detach works through DIE
    }
}

#-----------------------------------------------------------------------

sub log_change($@)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($c, @args) = @_;

    my $u = ($c->user_exists) ? $c->user->id : 'anonymous';

    eval {
        $c->model('Changelogs')->create($u, @args);
    };
    if ($@) {
        $c->log->error("Unable to update database changelog: '$@'");
    }
}

#-----------------------------------------------------------------------
# PRIVATE INTERFACE (HELPER METHODS)
#-----------------------------------------------------------------------

# _chcnf($$)
#
#  Usage     : $self->_chcnf($c, $conf);
#  Purpose   : Perform the check and preprocessing of action configurations
#  Arguments : CONTEXT $c - Catalyst context [MANDATORY]
#              HASHREF $conf - action configurations [MANDATORY]
#  Returns   : Nothing
#  Throws    : Dies, if not invoked on object instance
#  Comment   : Manipulates (normalizes) configurations

sub _chcnf($$)
{
    my ($self, $c, $conf, @required) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);
    confess "Configurations be given via hash reference" unless $conf and ref $conf eq 'HASH';

    # First check the presence of required configurations
    foreach my $k (@required) {
        confess "Missing mandatory configuration '$k'" unless exists $conf->{$k};
    }

    # Autofill the missing configurations
    unless ($conf->{CNF_CONTROLLER()}) {
        $conf->{CNF_CONTROLLER()} = $conf->{CNF_MODEL()};
    }
    unless ($conf->{CNF_ITEMS()}) {
        $conf->{CNF_ITEMS()} = $conf->{CNF_MODEL()};
    }
    unless ($conf->{CNF_ITEM()}) {
        $conf->{CNF_ITEM()} = $conf->{CNF_MODEL()};
        $conf->{CNF_ITEM()} =~ s/s$//;
    }
    unless ($conf->{CNF_ITEMS_S()}) {
        $conf->{CNF_ITEMS_S()} = $conf->{CNF_ITEMS()};
    }
    unless ($conf->{CNF_ITEM_S()}) {
        $conf->{CNF_ITEM_S()} = $conf->{CNF_ITEM()};
    }
    unless ($conf->{CNF_MENU()}) {
        $conf->{CNF_MENU()} = $conf->{CNF_MODEL()};
    }

    # Normalize the configurations
    $conf->{CNF_CONTROLLER()} = ucfirst($conf->{CNF_CONTROLLER()});
    $conf->{CNF_MODEL()}      = ucfirst($conf->{CNF_MODEL()});
    $conf->{CNF_ITEMS()}      = lc($conf->{CNF_ITEMS()});
    $conf->{CNF_ITEM()}       = lc($conf->{CNF_ITEM()});
    $conf->{CNF_ITEMS_S()}    = lc($conf->{CNF_ITEMS_S()});
    $conf->{CNF_ITEM_S()}     = lc($conf->{CNF_ITEM_S()});
    $conf->{CNF_MENU()}       = lc($conf->{CNF_MENU()});

    # Perform the validation
    foreach my $k (keys %$conf) {
        confess "Unknown action configuration '$k'\n" unless $VALID_CONFIGS{$k};
    }
}

# _choper($$$)
#
#  Usage     : $self->_choper($c, $type, $conf);
#  Purpose   : Perform the check if item ID
#  Arguments : CONTEXT $c - Catalyst context [MANDATORY]
#              ENUM $type - type of the error handler to use on failure [MANDATORY]
#              HASHREF $conf - action configurations [MANDATORY]
#              STRING $id - validated ID [MANDATORY]
#  Returns   : Nothing
#  Throws    : Dies, if not invoked on object instance
#  Comment   : Performs Catalyst detach() on check failure

sub _choper($$$)
{
    my ($self, $c, $type, $conf) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);
    confess "Error handler must be given as argument" unless $type;
    confess "Invalid error handler '$type'" unless $type eq HAWAT_ERROR_HANDLER or $type eq HAWAT_ERROR_HANDLER_R;

    # Both error handlers expect additional argument, but not the same
    my @arg = (($type eq HAWAT_ERROR_HANDLER) ? ('menu', $conf->{CNF_MENU()}) : ('backlink', $conf->{CNF_BACKLINK()}));

    # Check, that the model can perform the requested modification
    my $operation = $conf->{CNF_OPERATION()} || 'update';
    my $model = $c->model($conf->{CNF_MODEL()});
    unless ($model->can($operation)) {
        $c->detach($type, [message => ucfirst($conf->{CNF_ITEM()})." could not perform operation '$operation' due to application misconfiguration.", @arg]);
        return; # Only for readability, detach works through DIE
    }
    return ($model, $operation);
}

# _chid($$$$)
#
#  Usage     : $self->_chid($c, $type, $conf, $id);
#  Purpose   : Perform the check if item ID
#  Arguments : CONTEXT $c - Catalyst context [MANDATORY]
#              ENUM $type - type of the error handler to use on failure [MANDATORY]
#              HASHREF $conf - action configurations [MANDATORY]
#              STRING $id - validated ID [MANDATORY]
#  Returns   : Nothing
#  Throws    : Dies, if not invoked on object instance
#  Comment   : Performs Catalyst detach() on check failure

sub _chid($$$$)
{
    my ($self, $c, $type, $item, $conf, $id) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);
    confess "Error handler must be given as argument" unless $type;
    confess "Invalid error handler '$type'" unless $type eq HAWAT_ERROR_HANDLER or $type eq HAWAT_ERROR_HANDLER_R;
    confess "Item type must be given as argument" unless $item;

    # Both error handlers expect additional argument, but not the same
    my @arg = (($type eq HAWAT_ERROR_HANDLER) ? ('menu', $conf->{CNF_MENU()}) : ('backlink', $conf->{CNF_BACKLINK()}));

    # Perform ID sanity check: Item ID is mandatory argument
    unless ($id) {
        $c->detach($type, [message => ucfirst($conf->{$item})." ID was expected to be given as argument.", @arg]);
        return; # Only for readability, detach works through DIE
    }
    # Perform argument sanity check: Regexp check on ID is optional
    if ($conf->{CNF_REGEXP_ID()}) {
        my $re = $conf->{CNF_REGEXP_ID()};
        unless ($id =~ /$re/) {
            $c->detach($type, [message => "Invalid ".$conf->{$item}." ID '$id' received as argument.", @arg]);
            return; # Only for readability, detach works through DIE
        }
    }
}

# _chitem($$$$)
#
#  Usage     : $self->_chitem($c, $type, $conf, $id);
#  Purpose   : Perform the check if item ID
#  Arguments : CONTEXT $c - Catalyst context [MANDATORY]
#              ENUM $type - type of the error handler to use on failure [MANDATORY]
#              HASHREF $conf - action configurations [MANDATORY]
#              STRING $id - validated ID [MANDATORY]
#  Returns   : Nothing
#  Throws    : Dies, if not invoked on object instance
#  Comment   : Performs Catalyst detach() on check failure

sub _chitem($$$$)
{
    my ($self, $c, $type, $conf, $id, $modification, @args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);
    confess "Error handler must be given as argument" unless $type;
    confess "Invalid error handler '$type'" unless $type eq HAWAT_ERROR_HANDLER or $type eq HAWAT_ERROR_HANDLER_R;

    # Both error handlers expect additional argument, but not the same
    my @arg = (($type eq HAWAT_ERROR_HANDLER) ? ('menu', $conf->{CNF_MENU()}) : ('backlink', $conf->{CNF_BACKLINK()}));

    # Attempt to fetch the item from database
    my ($item, $count, $duration, @more) = $c->model($conf->{CNF_MODEL()})->fetch($id, @args);
    unless ($count) {
        $c->detach($type, [message => ucfirst($conf->{CNF_ITEM()})." '$id' was not found in database.", @arg]);
        return; # Only for readability, detach works through DIE
    }

    # Put the item to the stash
    $c->stash($conf->{CNF_ITEM()}.'_db' => $item);

    # Optionally dump interesting variables - dependent on the DEVEL switch
    $self->dump_variable($c, ucfirst($conf->{CNF_ITEM()})." before '$modification'", $item);

    return $item;
}

# _fetchSubitem($$$$)
#
#  Usage     : my $group, $count, $duration) = $c->model('Groups')->fetchSubitem($item, $type, $id);
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _fetchSubitem($$$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($c, $item, $conf, $id) = @_;
    die "Item must be given as argument\n" unless $item;
    die "Configurations must be given as argument\n" unless $conf;
    die "Subitem ID must be given as argument\n" unless $id;

    my $attr = $conf->{CNF_ATTRIBUTE()};
    $attr = $conf->{CNF_SUBITEM()} unless $attr;

    #$c->log->debug("FSI1: $attr");

    return undef unless $item->{$attr} and ref $item->{$attr} eq 'ARRAY';

    #$c->log->debug("FSI2: $attr");
    foreach my $i (@{$item->{$attr}}) {
        return $i if $i->{HAWAT_ATTR_MID()} and $i->{HAWAT_ATTR_MID()} eq $id;
    }
    return undef;
}

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
