package Hawat::Controller::Event;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Data::Dumper;
use Mentat::Core qw(:c_hawat);

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

BEGIN { extends 'Hawat::Controller::Secure'; }

__PACKAGE__->config(
    class       => 'Hawat::Controller::Event',
    constructor => 'new',
);

=head1 NAME

Hawat::Controller::Event - Catalyst controller for event management

=head1 DESCRIPTION

Catalyst controller for handling actions related to event management.

=head1 ACTIONS

=cut

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

=head2 index

Default controller action, redirects to the 'list' action

=cut

sub index :Path :Args(0) {
    my ( $self, $c ) = @_;
    $c->response->redirect($c->uri_for($self->action_for('list')));
}

=head2 list

View the list of all known events stored

=cut

sub list :Local :Args(0) {
    my ($self, $c) = @_;

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/find' action
    my $conf = $self->_dbicnf(
        limit  => $c->cfg('result_limit'),
        sortby => 'id',
        sort   => 'asc',
    );
    $c->forward(HAWAT_DBI_FIND, [$conf, $c]);

    if ($c->stash->{events_count}) {
        my ($example, $error);
        foreach my $e (@{$c->stash->{events}}) {
            ($example, $error) = $c->model('Events')->generate_idea($e);
            $e->{error_idea} = $error;
        }
    }

    # TODO: make use of 'nowrap' for ajax queries
    #$c->stash(nowrap => 1);

    # Optionally dump interesting variables - dependent on the DEVEL switch
    #$self->dump_variable($c, 'Field', $variable) if $c->cfg('devel');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 view

View the details of arbitrary event

=cut

#sub view :Local :Args(1) { # Switch to let the framework handle the error reporting
sub view :Local {
    my ($self, $c, $id) = @_;

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/fetch' action
    my $conf = $self->_dbicnf();
    $c->forward(HAWAT_DBI_FETCH, [$conf, $id]);

    # Configure and forward to '/dbitem/find' action
    my $conf_find = $self->_dbicnf(
        sortby => 'id',
        sort   => 'asc',
    );
    $c->forward(HAWAT_DBI_FIND, [$conf_find, $c]);

    # If the event was successfully fetched, generate the example messages
    if ($c->stash->{event_count}) {
        my ($example, $error, $email);

        # Attempt to generate example IDEA message
        ($example, $error) = $c->model('Events')->generate_idea($c->stash->{event});
        $c->stash(example_idea => $example->to_string(1)) if $example;
        $c->stash(error_idea   => $error) if $error;

        # Attempt to generate email bodies
        $email = $c->model('Events')->generate_email($c->stash->{event}, $example);
        $c->stash(example_email => $email) if $email;
    }

    my $tree_view = $c->model('Events')->tree_view($c->stash->{events}, 'category');
    $c->stash(events_tree => $tree_view);

    #my $tree_view_cls = $c->model('Events')->tree_view($c->stash->{events}, 'analyzer', 'class');
    my $tree_view_cls = $c->model('Events')->tree_view($c->stash->{events}, 'class');
    $c->stash(events_tree_cls => $tree_view_cls);

    my $tree_view_sev = $c->model('Events')->tree_view($c->stash->{events}, 'severity');
    $c->stash(events_tree_sev => $tree_view_sev);

    my $events = $c->model('Events')->list();
    $c->stash(events_list => $events);

    # TODO: make use of 'nowrap' for ajax queries
    #$c->stash(nowrap => 1);

    # Optionally dump interesting variables - dependent on the DEVEL switch
    #$self->dump_variable($c, 'Field', $variable) if $c->cfg('devel');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 browse

Browse the list of all known events stored

=cut

#sub browse :Local :Args(1) { # Switch to let the framework handle the error reporting
sub browse :Local {
    my ($self, $c, $id) = @_;

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/find' action
    my $conf_find = $self->_dbicnf(
        sortby => 'id',
        sort   => 'asc',
    );
    $c->forward(HAWAT_DBI_FIND, [$conf_find, $c]);

    if ($id) {
        # Configure and forward to '/dbitem/fetch' action
        my $conf = $self->_dbicnf();
        $c->forward(HAWAT_DBI_FETCH, [$conf, $id]);
    }
    else {
        $c->stash->{event} = $c->stash->{events}->[0];
        $c->stash->{event_count} = ($c->stash->{event}) ? 1 : 0;
    }

    # If the event was successfully fetched, generate the example messages
    if ($c->stash->{event_count}) {
        my ($example, $error, $email);

        # Attempt to generate example IDEA message
        ($example, $error) = $c->model('Events')->generate_idea($c->stash->{event});
        $c->stash(example_idea => $example->to_string(1)) if $example;
        $c->stash(error_idea   => $error) if $error;

        # Attempt to generate email bodies
        $email = $c->model('Events')->generate_email($c->stash->{event}, $example);
        $c->stash(example_email => $email) if $email;
    }

    my $tree_view = $c->model('Events')->tree_view($c->stash->{events}, 'category');
    $c->stash(events_tree => $tree_view);

    #my $tree_view_cls = $c->model('Events')->tree_view($c->stash->{events}, 'analyzer', 'class');
    my $tree_view_cls = $c->model('Events')->tree_view($c->stash->{events}, 'class');
    $c->stash(events_tree_cls => $tree_view_cls);

    my $tree_view_sev = $c->model('Events')->tree_view($c->stash->{events}, 'severity');
    $c->stash(events_tree_sev => $tree_view_sev);

    my $events = $c->model('Events')->list();
    $c->stash(events_list => $events);

    # TODO: make use of 'nowrap' for ajax queries
    #$c->stash(nowrap => 1);

    # Optionally dump interesting variables - dependent on the DEVEL switch
    #$self->dump_variable($c, 'Field', $variable) if $c->cfg('devel');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 import

Import events from external file

=cut

sub import :Local {
    my ($self, $c) = @_;

    my ($result, $errors) = $c->model('Events')->import();

    foreach my $r (@$result) {
        $self->flash_success($c, "Inserted template ".$r);
    }
    foreach my $e (@$errors) {
        $self->flash_error($c, $e);
    }

    $c->response->redirect($c->uri_for($c->controller('Event')->action_for('list')));
}

=head2 export

Export events to external file

=cut

sub export :Local {
    my ($self, $c) = @_;

    my $json = $c->model('Events')->export();
    if ($json) {
        $c->response->content_type('application/json; charset=utf-8');
        $c->response->header('Content-Disposition' => 'attachment; filename="msgtmplts.json"');
        $c->response->body($json);
    }
    else {

        my $error = <<'END_OF_HTML';
<html>
<head><title>503 Database Returned No Results</title></head>
<body>
    <h1>Error</h1>
    <p>Sorry, the database returned no results. Please try again later.</p>
</body>
</html>
END_OF_HTML

        $c->response->content_type('text/html; charset=utf-8');
        $c->response->status(503);
        $c->response->body($error);
    }
}

=head2 validate

Validate all defined events against the IDEA schema

=cut

sub validate :Local {
    my ($self, $c) = @_;

    my ($events, $count, $count_all, $duration, $query) = $c->model('Events')->find(undef, undef, undef, 'id', 'asc');

    if ($count) {
        my ($result, $errors) = $c->model('Events')->validate_template(@$events);
        if (scalar @$result) {
            $self->flash_success($c, "All event templates for creating IDEA messages are valid.");
            #foreach my $r (@$result) {
            #   $self->flash_success($c, $r);
            #}
        }
        else {
            foreach my $e (@$errors) {
                $self->flash_error($c, $e);
            }
        }
    }
    else {
        $self->flash_error($c, "There are currently no events defined in the database.");
    }

    $c->response->redirect($c->uri_for($c->controller('Event')->action_for('list')));
}

=head2 create

Create new event

=cut

sub create :Local {
    my ($self, $c) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/event');

    # Configure and forward to '/dbitem/create' action
    my $conf = $self->_dbicnf(
        backlink => $backlink,
    );
    $c->forward(HAWAT_DBI_CREATE, [$conf, $c]);

    # If the event was successfully fetched, generate the example messages
    my ($example, $error);
    if ($c->stash->{event_form}->{rules_idea} and
        ref $c->stash->{event_form}->{rules_idea} eq 'ARRAY' and
        scalar @{$c->stash->{event_form}->{rules_idea}}) {
        # Attempt to generate example IDEA message
        ($example, $error) = $c->model('Events')->generate_idea($c->stash->{event_form}, $c);
        $c->stash(example_idea => $example->to_string(1)) if $example;
        $c->stash(error_idea => $error) if $error;
    }

    $c->stash(severities => $c->model('Events')->severities());

    # Choose the common create+update template
    $c->stash(template => 'event/creatupdate');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 update

Update existing event

=cut

#sub update :Local :Args(1) { # Switch to let the framework handle the error reporting
sub update :Local {
    my ($self, $c, $id) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/event');

    # Configure and forward to '/dbitem/update' action
    my $conf = $self->_dbicnf(
        backlink => $backlink,
    );
    $c->forward(HAWAT_DBI_UPDATE, [$conf, $id, $c]);

    # If the event was successfully fetched, generate the example messages
    my ($example, $error);
    if ($c->stash->{event_form}->{rules_idea} and
        ref $c->stash->{event_form}->{rules_idea} eq 'ARRAY' and
        scalar @{$c->stash->{event_form}->{rules_idea}}) {
        # Attempt to generate example IDEA message
        ($example, $error) = $c->model('Events')->generate_idea($c->stash->{event_form}, $c);
        $c->stash(example_idea => $example->to_string(1)) if $example;
        $c->stash(error_idea => $error) if $error;
    }

    $c->stash(severities => $c->model('Events')->severities());

    # Choose the common create+update template
    $c->stash(template => 'event/creatupdate');

    # Optionally dump interesting variables - dependent on the DEVEL switch
    #$self->dump_variable($c, 'Field', $variable) if $c->cfg('devel');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 rename

Rename existing event

=cut

#sub rename :Local :Args(1) { # Switch to let the framework handle the error reporting
sub rename :Local {
    my ($self, $c, $id) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/event');

    # Configure and forward to '/dbitem/rename' action
    my $conf = $self->_dbicnf(
        backlink => $backlink,
    );
    $c->forward(HAWAT_DBI_RENAME, [$conf, $id, $c]);

    # Use the general rename form
    $c->stash(template    => 'inc/dialog_rename.html.tt',
              title       => 'Mentat - Rename event',
              rename_item => 'Event');

    # Optionally dump interesting variables - dependent on the DEVEL switch
    #$self->dump_variable($c, 'Field', $variable) if $c->cfg('devel');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 delete

Delete one particular event

=cut

#sub delete :Local :Args(1) {# Switch to let the framework handle the error reporting
sub delete :Local {
    my ($self, $c, $id, $confirm) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/event');

    # Configure and forward to '/dbitem/delete' action
    my $conf_d = $self->_dbicnf(
        backlink   => $backlink,
        backlink_d => $c->uri_for('/event/list'),
    );
    $c->forward(HAWAT_DBI_DELETE, [$conf_d, $id, $confirm]);

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 remove

Remove multiple events

=cut

#sub remove :Local :Args(1) {# Switch to let the framework handle the error reporting
sub remove :Local {
    my ($self, $c, $id, $confirm) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/event');

    # Configure and forward to '/dbitem/delete' action
    my $conf_d = $self->_dbicnf(
        backlink   => $backlink,
        backlink_d => $c->uri_for('/event/list'),
    );
    $c->forward(HAWAT_DBI_REMOVE, [$conf_d, $id, $confirm]);

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 enable

Enable event

=cut

#sub enable :Local :Args(2) { # Switch to let the framework handle the error reporting
sub enable :Local {
    my ($self, $c, $id, $confirm) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/event');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'enable',
        backlink  => $backlink,
    );
    if ($id) {
        $conf->{msg_success} = "Successfully enabled event '$id'.";
        $conf->{msg_failure} = "Unable to enable event '$id'.";
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id]);

    # Optionally dump interesting variables - dependent on the DEVEL switch
    #$self->dump_variable($c, 'Field', $variable) if $c->cfg('devel');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 disable

Disable event

=cut

#sub disable :Local :Args(2) { # Switch to let the framework handle the error reporting
sub disable :Local {
    my ($self, $c, $id, $confirm) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/event');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'disable',
        backlink  => $backlink,
    );
    if ($id) {
        $conf->{msg_success} = "Successfully disabled event '$id'.";
        $conf->{msg_failure} = "Unable to disable event '$id'.";
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id]);

    # Optionally dump interesting variables - dependent on the DEVEL switch
    #$self->dump_variable($c, 'Field', $variable) if $c->cfg('devel');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

# _dbicnf(%)
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _dbicnf(%)
{
    my ($self, %cnf_a) = @_;
    my $cnf = {
            controller => 'Event',
            model      => HAWAT_MODEL_EVENTS(),
            items      => 'Events',
            item       => 'Event',
            menu       => 'admin',
        };
    foreach my $k (keys %cnf_a) {
        $cnf->{$k} = $cnf_a{$k};
    }
    return $cnf;
}

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
