package Hawat::Controller::Group;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Data::Dumper;
use Mentat::Core qw(:c_hawat);
use Data::Filter::Formater;

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

BEGIN { extends 'Hawat::Controller::Secure'; }

__PACKAGE__->config(
    class       => 'Hawat::Controller::Group',
    constructor => 'new',
);

use constant SELECTORS_CACHE_REWRITE => 0; # Grow up mongodb performance !!!

=head1 NAME

Hawat::Controller::Group - Catalyst controller for group management

=head1 DESCRIPTION

Catalyst controller for handling actions related to group management.

=head1 ACTIONS

=cut

my $PERIODS = [
    'none',
    '10mins',
    '15mins',
    '20mins',
    '30mins',
    '1hr',
    '2hrs',
    '3hrs',
    '4hrs',
    '6hrs',
    '8hrs',
    '12hrs',
    '1day',
    '2days',
    '3days',
    '4days',
    '5days',
    '6days',
    '7days',
    '8days',
    '9days',
    '10days',
    '1week',
    '2weeks',
    '3weeks',
    '4weeks',
];

my $PERIODS_H = {
    'none'   => 0,
    '10mins' => 600,
    '15mins' => 900,
    '20mins' => 1200,
    '30mins' => 1800,
    '1hr'    => 3600,
    '2hrs'   => (3600 * 2),
    '3hrs'   => (3600 * 3),
    '4hrs'   => (3600 * 4),
    '6hrs'   => (3600 * 6),
    '8hrs'   => (3600 * 8),
    '12hrs'  => (3600 * 12),
    '1day'   => 86400,
    '2days'  => (86400 * 2),
    '3days'  => (86400 * 3),
    '4days'  => (86400 * 4),
    '5days'  => (86400 * 5),
    '6days'  => (86400 * 6),
    '7days'  => (86400 * 7),
    '8days'  => (86400 * 8),
    '9days'  => (86400 * 9),
    '10days' => (86400 * 10),
    '1week'  => 604800,
    '2weeks' => (604800 * 2),
    '3weeks' => (604800 * 3),
    '4weeks' => (604800 * 4),
};

my $TIMING_THRS = {
    'low'      => { 'period' =>  { 'l' =>  '10mins', 'h' =>  '1day' },   'threshold' =>  { 'l' =>  'none', 'h' =>  '7days' }, 'relapse' =>  { 'l' =>  'none', 'h' =>  '2days' } },
    'medium'   => { 'period' =>  { 'l' =>  '10mins', 'h' =>  '1day' },   'threshold' =>  { 'l' =>  'none', 'h' =>  '7days' }, 'relapse' =>  { 'l' =>  'none', 'h' =>  '2days' } },
    'high'     => { 'period' =>  { 'l' =>  '10mins', 'h' =>  '4hrs' },   'threshold' =>  { 'l' =>  'none', 'h' =>  '7days' }, 'relapse' =>  { 'l' =>  'none', 'h' =>  '2days' } },
    'critical' => { 'period' =>  { 'l' =>  '10mins', 'h' =>  '10mins' }, 'threshold' =>  { 'l' =>  'none', 'h' =>  '7days' }, 'relapse' =>  { 'l' =>  'none', 'h' =>  '2days' } },
};

my $TIMING_DFLT = {
    'low'      => { 'period' =>  '1day',   'threshold' =>  '7days', 'relapse' =>  '2days' },
    'medium'   => { 'period' =>  '2hrs',   'threshold' =>  '2days', 'relapse' =>  '1day' },
    'high'     => { 'period' =>  '10mins', 'threshold' =>  '2hrs',  'relapse' =>  'none' },
    'critical' => { 'period' =>  '10mins', 'threshold' =>  'none',  'relapse' =>  'none' },
};

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

=head2 index

Default controller action, redirects to the 'list' action

=cut

sub index :Path :Args(0) {
    my ( $self, $c ) = @_;
    $c->response->redirect($c->uri_for($self->action_for('list')));
}

=head2 list

View the list of all reports stored in the database

=cut

sub list :Local :Args(0) {
    my ($self, $c) = @_;

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/find' action
    my $conf = $self->_dbicnf(
        limit  => $c->cfg('result_limit'),
        sortby => 'id',
        sort   => 'asc',
    );
    $c->forward(HAWAT_DBI_FIND, [$conf]);
}

=head2 view

View the details of arbitrary group

=cut

#sub view :Local :Args(1) { # Switch to let the framework handle the error reporting
sub view :Local {
    my ($self, $c, $id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/fetch' action
    my $conf = $self->_dbicnf();
    $c->forward(HAWAT_DBI_FETCH, [$conf, $id]);

    # Configure and forward to '/dbitem/find' action
    my $conf_r = $c->controller('Report')->_dbicnf(
        search => $id,
        limit  => 5,
        sortby => 'ts',
        sort   => 'desc',
    );
    $c->forward(HAWAT_DBI_FIND, [$conf_r, $c->check_roles('admin'), $c->get_memberships(), $c->cfg('reports_dir')]);

    # Add custom data to the stash
    my $users     = $c->model('Users')->list();
    my $groups    = $c->model('Groups')->list();
    my $members   = $c->model('Users')->listGroupMembers($id);
    my %members_h = map { $_ => 1 } @$members;
    $c->stash(users     => $users);
    $c->stash(groups    => $groups);
    $c->stash(members   => $members);
    $c->stash(members_h => \%members_h);

    $c->stash($c->model('Groups')->get_timing_consts());
}

=head2 dashboard

View the group dashboard

=cut

#sub view :Local :Args(1) { # Switch to let the framework handle the error reporting
sub dashboard :Local {
    my ($self, $c, $id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/fetch' action
    my $conf = $self->_dbicnf();
    $c->forward(HAWAT_DBI_FETCH, [$conf, $id]);

    # Configure and forward to '/dbitem/find' action
    my $conf_r = $c->controller('Report')->_dbicnf(
        search => $id,
        sortby => 'ts',
        sort   => 'desc',
        dfrom  => $c->time_before('2weeks', 1),
    );
    $c->forward(HAWAT_DBI_FIND, [$conf_r, $c->check_roles('admin'), $c->get_memberships(), $c->cfg('reports_dir')]);

    my $fr = $c->stash->{reports};
    eval {
        $c->stash(frs => $c->model('Reports')->calculate_frs($fr, 'single'));
    };
    if ($@) {
        $self->flash_error($c, "Error calculating FRS: $@");
    }

    #$c->stash($c->model('Groups')->get_timing_consts());
}

=head2 create

Create new group

=cut

sub create :Local {
    my ($self, $c) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/create' action
    my $conf = $self->_dbicnf(
        backlink => $backlink,
    );
    $c->forward(HAWAT_DBI_CREATE, [$conf]);

    $c->stash($c->model('Groups')->get_timing_consts());

    # Choose the common create+update template
    $c->stash(template => 'group/creatupdate');
}

=head2 update

Update existing group

=cut

#sub update :Local :Args(1) { # Switch to let the framework handle the error reporting
sub update :Local {
    my ($self, $c, $id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/update' action
    my $conf = $self->_dbicnf(
        backlink => $backlink,
    );
    $c->forward(HAWAT_DBI_UPDATE, [$conf, $id, $c]);

    $c->stash($c->model('Groups')->get_timing_consts());

    # Choose the common create+update template
    $c->stash(template => 'group/creatupdate');
}

=head2 rename

Rename existing group

=cut

#sub rename :Local :Args(1) { # Switch to let the framework handle the error reporting
sub rename :Local {
    my ($self, $c, $id) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/rename' action
    my $conf = $self->_dbicnf(
        backlink => $backlink,
    );
    $c->forward(HAWAT_DBI_RENAME, [$conf, $id]);

    # Use the general rename form
    $c->stash(template    => 'inc/dialog_rename.html.tt',
              title       => 'Hawat - Rename group',
              rename_item => 'Group');
}

=head2 delete

Delete one particular group

=cut

#sub delete :Local :Args(1) {# Switch to let the framework handle the error reporting
sub delete :Local {
    my ($self, $c, $id, $confirm) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/delete' action
    my $conf_d = $self->_dbicnf(
        backlink   => $backlink,
        backlink_d => $c->uri_for('/group/list'),
    );
    $c->forward(HAWAT_DBI_DELETE, [$conf_d, $id, $confirm]);

    # Configure and forward to '/dbitem/modify' action
    my $conf_m = {
        controller   => 'Users',
        model        => 'Users',
        operation => 'deleteGroupRefs',
        item         => 'User',
        backlink     => $c->uri_for('/group/list'),
        multi        => 1,
        msg_success  => "Successfully removed all references to group '$id' from all user accounts.",
        msg_none     => "There were no references to group '$id' in user account records.",
        msg_failure  => "Unable to remove all references to group '$id' from all user accounts.",
    };
    $c->forward(HAWAT_DBI_MODIFY, [$conf_m, $id]);
}

=head2 addManager

Add given role to one particular user account

=cut

#sub addManager :Local :Args(2) {# Switch to let the framework handle the error reporting
sub addManager :Local {
    my ($self, $c, $id, $manager) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'addManager',
        backlink  => $backlink,
    );
    if ($id and $manager) {
        $conf->{msg_success} = "Successfully added management rights for user account '$manager' to group '$id'.";
        $conf->{msg_failure} = "Unable to add management rights for user account '$manager' to group '$id'.";
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id, $manager]);
}

=head2 removeManager

Remove given group from one particular user account

=cut

#sub removeManager :Local :Args(2) {# Switch to let the framework handle the error reporting
sub removeManager :Local {
    my ($self, $c, $id, $manager) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'removeManager',
        backlink  => $backlink,
    );
    if ($id and $manager) {
        $conf->{msg_success} = "Successfully removed management rights for user account '$manager' from group '$id'.";
        $conf->{msg_failure} = "Unable to remove management rights for user account '$manager' from group '$id'.";
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id, $manager]);
}

=head2 removeQuery

Remove given stored query from one particular group

=cut

#sub removeQuery :Local :Args(2) {# Switch to let the framework handle the error reporting
sub removeQuery :Local {
    my ($self, $c, $id, $query) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/alert');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'removeQuery',
        backlink  => $backlink,
    );
    if ($id and $query) {
        $conf->{msg_success} = "Successfully removed stored query '$query' from group '$id'.";
        $conf->{msg_failure} = "Unable to remove rstored query '$query' from group '$id'.";
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id, $query]);
}

=head2 addFilter

Create new reporting filter

=cut

#sub addFilter :Local :Args(1) { # Switch to let the framework handle the error reporting
sub addFilter :Local {
    my ($self, $c, $id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Load necessary item sets
    $c->forward('loadItemSets');
    
    # Configure and forward to '/dbitem/addFilter' action
    my $conf = $self->_dbicnf(
        backlink  => $backlink,
        operation => 'addFilter',
        subitem   => 'filter',
    );
    $c->forward(HAWAT_DBI_SCREATE, [$conf, $id, $c->stash->{'eventlib_index'}]);
    
    # Generate filter structure graph
    if ($c->stash->{filter_form} and $c->stash->{filter_form}->{filter}) {
        eval {
            my ($flt, $err) = $c->model('Groups')->parseFilter($c->stash->{filter_form}->{filter});
            my $frm = Data::Filter::Formater->new();
            my $rsl = $frm->format_html($flt);
            $c->stash(filter_tree => $rsl);
        };
        if ($@) {
            $self->flash_error($c, "Unable to generate filter tree structure.");
        }
    }
    my ($fltl, $cnt) = $c->model('Groups')->listGroupFilters($id);
    $c->stash(group_filters => $fltl);

    # Choose the common create+updateFilter template
    $c->stash(template => 'group/creatupdate-f');
}

=head2 updateFilter

Create new reporting filter

=cut

#sub updateFilter :Local :Args(2) { # Switch to let the framework handle the error reporting
sub updateFilter :Local {
    my ($self, $c, $id, $filter_id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Load necessary item sets
    $c->forward('loadItemSets');
    
    # Configure and forward to '/dbitem/updateFilter' action
    my $conf = $self->_dbicnf(
        backlink  => $backlink,
        operation => 'updateFilter',
        subitem   => 'filter',
        attribute => 'rep_filters',
    );
    $c->forward(HAWAT_DBI_SUPDATE, [$conf, $id, $filter_id, $c->stash->{'eventlib_index'}]);
    
    # Generate filter structure graph
    if ($c->stash->{filter_form} and $c->stash->{filter_form}->{filter}) {
        eval {
            my ($flt, $error) = $c->model('Groups')->parseFilter($c->stash->{filter_form}->{filter});
            my $frm = Data::Filter::Formater->new();
            my $rsl = $frm->format_html($flt);
            $c->stash(filter_tree => $rsl);
        };
        if ($@) {
            $self->flash_error($c, "Unable to generate filter tree structure.");
        }
    }
    my ($fltl, $cnt) = $c->model('Groups')->listGroupFilters($id);
    $c->stash(group_filters => $fltl);

    # Choose the common create+updateFilter template
    $c->stash(template => 'group/creatupdate-f');
}

=head2 removeFilter

Create new reporting filter

=cut

#sub removeFilter :Local :Args(2) { # Switch to let the framework handle the error reporting
sub removeFilter :Local {
    my ($self, $c, $id, $filter_id, $confirm) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/removeFilter' action
    my $conf = $self->_dbicnf(
        action    => 'removeFilter',
        backlink  => $backlink,
        operation => 'removeFilter',
        subitem   => 'filter',
    );
    $c->forward(HAWAT_DBI_SDELETE, [$conf, $id, $filter_id, $confirm, $c]);
}

=head2 enableFilter

Enable reporting filter

=cut

#sub enableFilter :Local :Args(2) { # Switch to let the framework handle the error reporting
sub enableFilter :Local {
    my ($self, $c, $id, $filter_id, $confirm) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/enableFilter' action
    my $conf = $self->_dbicnf(
        action => 'enableFilter',
        backlink => $backlink,
        operation => 'enableFilter',
        subitem => 'filter',
    );
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id, $filter_id, $confirm, $c]);
}

=head2 disableFilter

Disable reporting filter

=cut

#sub disableFilter :Local :Args(2) { # Switch to let the framework handle the error reporting
sub disableFilter :Local {
    my ($self, $c, $id, $filter_id, $confirm) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/disableFilter' action
    my $conf = $self->_dbicnf(
        action => 'disableFilter',
        backlink => $backlink,
        operation => 'disableFilter',
        subitem => 'filter',
    );
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id, $filter_id, $confirm, $c]);
}

=head2 testFilters

Test group filters on given message

=cut

sub testFilters :Local {
    my ($self, $c, $id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));
    
    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    my $message_id = $c->request->params->{message_id};
    $c->stash(group_id   => $id);
    $c->stash(message_id => $message_id);

    if ($message_id) {
        my ($message, $count, $duration) = $c->model('Alerts')->fetch($message_id);
        $c->stash(message => $message);

        if ($message) {
            my $result = $c->model('Groups')->test_filters($id, $message);
            $c->stash(test_result => $result);
            $self->dump_variable($c, 'Filtering result', $result) if $c->cfg('devel');
        }
        else {
            $c->stash(test_result => {'str' => "There is no message with ID '$message_id'"});
        }
    }

    # Choose the common create+update template
    $c->stash(template => 'group/filters_test');
}

=head2 addNetwork

Create new network record

=cut

#sub addNetwork :Local :Args(1) { # Switch to let the framework handle the error reporting
sub addNetwork :Local {
    my ($self, $c, $id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Determine the type of the action
    my $action = $c->request->params->{submit};

    # Perform whois query
    $c->forward('whoisQuery');
    
    # Configure and forward to '/dbitem/addNetwork' action
    my $conf = $self->_dbicnf(
        backlink => $backlink,
        operation => 'addNetwork',
        subitem => 'network',
    );
    $c->forward(HAWAT_DBI_SCREATE, [$conf, $id]);

    if ($action eq HAWAT_FORM_AUTOFILL()) {
        $c->stash->{'network_form'}->{'netname'} = $c->stash->{'whois_query'}->{'netname'};
    }
    
    # Choose the common create+updateNetwork template
    $c->stash(template => 'group/creatupdate-n');
}

=head2 updateNetwork

Update existing network record

=cut

#sub updateNetwork :Local :Args(2) { # Switch to let the framework handle the error reporting
sub updateNetwork :Local {
    my ($self, $c, $id, $network_id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Determine the type of the action
    my $action = $c->request->params->{submit};

    # Perform whois query
    $c->forward('whoisQuery');
    
    # Configure and forward to '/dbitem/updateNetwork' action
    my $conf = $self->_dbicnf(
        backlink => $backlink,
        operation => 'updateNetwork',
        subitem => 'network',
        attribute => 'networks',
    );
    $c->forward(HAWAT_DBI_SUPDATE, [$conf, $id, $network_id]);

    if ($action eq HAWAT_FORM_AUTOFILL()) {
        $c->stash->{'network_form'}->{'netname'} = $c->stash->{'whois_query'}->{'netname'};
    }
    
    # Choose the common create+updateNetwork template
    $c->stash(template => 'group/creatupdate-n');
}

=head2 removeNetwork

Remove network record

=cut

#sub removeNetwork :Local :Args(2) { # Switch to let the framework handle the error reporting
sub removeNetwork :Local {
    my ($self, $c, $id, $network_id, $confirm) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/removeNetwork' action
    my $conf = $self->_dbicnf(
        action => 'removeNetwork',
        backlink => $backlink,
        operation => 'removeNetwork',
        subitem => 'network',
    );
    $c->forward(HAWAT_DBI_SDELETE, [$conf, $id, $network_id, $confirm]);
}

=head2 checkNetworks

Check group networks for completeness

=cut

sub checkNetworks :Local {
    my ($self, $c, $id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Configure and forward to '/dbitem/fetch' action
    my $conf = $self->_dbicnf();
    $c->forward(HAWAT_DBI_FETCH, [$conf, $id]);
    
    # Fetch the form data (submitted using POST method)
    my $form = $c->request->body_params;
    $c->stash(network_list => $form->{network_list});
    
    # If the form was correctly submitted, attempt to insert the item to the database
    if ($form->{submitted} and $form->{network_list}) {
        my $result = $c->model('Groups')->check_networks($c->stash->{group}, $form->{network_list});
        $c->stash(check_result => $result);
        $self->dump_variable($c, 'Checking result', $result) if $c->cfg('devel');
    }
        
    # Choose the common create+update template
    $c->stash(template => 'group/networks_check');
}

=head2 mute

Mute email reporting for this group

=cut

#sub mute :Local :Args(1) {# Switch to let the framework handle the error reporting
sub mute :Local {
    my ($self, $c, $id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'mute',
        backlink     => $backlink,
        multi        => 1,
    );
    if ($id) {
        $conf->{msg_success}  = "Successfully muted email reporting for group(s) '$id'.",
        $conf->{msg_none}     = "There were no groups in need of muting email reporting.",
        $conf->{msg_failure}  = "Unable to mute email reporting for group(s) '$id'.",
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id]);
}

=head2 unmute

Unmute email reporting for this group

=cut

#sub unmute :Local :Args(1) {# Switch to let the framework handle the error reporting
sub unmute :Local {
    my ($self, $c, $id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'unmute',
        backlink  => $backlink,
        multi     => 1,
    );
    if ($id) {
        $conf->{msg_success}  = "Successfully unmuted email reporting for group(s) '$id'.",
        $conf->{msg_none}     = "There were no groups in need of unmuting email reporting.",
        $conf->{msg_failure}  = "Unable to unmute email reporting for group(s) '$id'.",
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id]);

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 redirect

Redirect email reporting for this group

=cut

#sub redirect :Local :Args(1) {# Switch to let the framework handle the error reporting
sub redirect :Local {
    my ($self, $c, $id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'redirect',
        backlink     => $backlink,
        multi        => 1,
    );
    if ($id) {
        $conf->{msg_success}  = "Successfully redirected email reporting for group(s) '$id'.",
        $conf->{msg_none}     = "There were no groups in need of email reporting redirection.",
        $conf->{msg_failure}  = "Unable to redirect email reporting for group(s) '$id'.",
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id]);

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 unredirect

Unredirect email reporting for this group

=cut

#sub unredirect :Local :Args(1) {# Switch to let the framework handle the error reporting
sub unredirect :Local {
    my ($self, $c, $id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'unredirect',
        backlink     => $backlink,
        multi        => 1,
    );
    if ($id) {
        $conf->{msg_success}  = "Successfully stopped email reporting redirection for group(s) '$id'.",
        $conf->{msg_none}     = "There were no groups in need of stopping email reporting redirection.",
        $conf->{msg_failure}  = "Unable to stop email reporting redirection for group(s) '$id'.",
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id]);

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 set_compression

Redirect email reporting for this group

=cut

#sub set_compression :Local :Args(1) {# Switch to let the framework handle the error reporting
sub set_compression :Local {
    my ($self, $c, $id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'set_compression',
        backlink     => $backlink,
        multi        => 1,
    );
    if ($id) {
        $conf->{msg_success}  = "Successfully set email attachment compression for group(s) '$id'.",
        $conf->{msg_none}     = "There were no groups in need of seting email attachment compression.",
        $conf->{msg_failure}  = "Unable to set email attachment compression for group(s) '$id'.",
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id]);

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 unset_compression

Unredirect email reporting for this group

=cut

#sub unset_compression :Local :Args(1) {# Switch to let the framework handle the error reporting
sub unset_compression :Local {
    my ($self, $c, $id) = @_;

    # Only admin can view any accounts, other users may view only their own
    $c->detach('access_denied') unless ($c->check_acl({g => $id}));

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/group');

    # Configure and forward to '/dbitem/modify' action
    my $conf = $self->_dbicnf(
        operation => 'unset_compression',
        backlink     => $backlink,
        multi        => 1,
    );
    if ($id) {
        $conf->{msg_success}  = "Successfully unset email attachment compression for group(s) '$id'.",
        $conf->{msg_none}     = "There were no groups in need of unseting email attachment compression.",
        $conf->{msg_failure}  = "Unable to unset email attachment compression for group(s) '$id'.",
    }
    $c->forward(HAWAT_DBI_MODIFY, [$conf, $id]);

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

#-------------------------------------------------------------------------------

sub loadItemSets :Private {
    my ($self, $c) = @_;

    my $item_sets;
    eval {
        $item_sets = $c->model(HAWAT_MODEL_GROUPS())->get_item_sets(SELECTORS_CACHE_REWRITE());

        # Put the retrieved data to stash
        $c->stash->{'itemset_nodenames'} = $item_sets->{Mentat::Core::KEY_CACHE_NODENAMES()};
        $c->stash->{'eventlib_index'}    = $item_sets->{Mentat::Core::KEY_CACHE_EVENTLIB()};

        $c->stash->{'itemset_stat_nodenames'}  = $item_sets->{'itemset-stat-nodenames'};
        $c->stash->{'itemset_stat_nodesws'}    = $item_sets->{'itemset-stat-nodesws'};
        $c->stash->{'itemset_stat_detectors'}  = $item_sets->{'itemset-stat-detectors'};
        $c->stash->{'itemset_stat_categories'} = $item_sets->{'itemset-stat-categories'};
    };
    if ($@) { $self->flash_error($c, $@); }
}

sub whoisQuery :Private {
    my ($self, $c) = @_;

    eval {
        my ($type, $result, $duration) = $c->model('Whois')->search($c->request->params->{network});

        # Put the retrieved data to stash
        if ($result->{remote}) {
            my $r = $result->{remote}->[0];
            $c->stash('whois_query' => $r);
        }
    };
    if ($@) { $self->flash_error($c, $@); }
}

# _dbicnf(%)
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _dbicnf(%)
{
    my ($self, %cnf_a) = @_;
    my $cnf = {
            controller => 'Group',
            model      => HAWAT_MODEL_GROUPS(),
            items      => 'Groups',
            item       => 'Group',
            menu       => 'admin',
        };
    foreach my $k (keys %cnf_a) {
        $cnf->{$k} = $cnf_a{$k};
    }
    return $cnf;
}

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
