package Hawat::Model::Briefs;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Carp;
use Scalar::Util 'blessed';
use Time::HiRes qw (time);

use Log::Loger;
use DNS::Resolver;
use Value::Convertor;
use Mentat::Stats::Brief;
use Mentat::Storage::Mongo;
use Mentat::Mailer::Report;

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

BEGIN { extends 'Catalyst::Model::Adaptor'; }

__PACKAGE__->config(
    class           => 'Hawat::Model::Briefs',
    constructor     => 'new',
    devel           => 0,
    database_name   => 'mentat',
    collection_name => 'briefs',
    test_mode       => 0,
);
has "devel"           => (is => 'ro', isa => 'Bool', required => 1);
has "database_name"   => (is => 'ro', isa => 'Str',  required => 1);
has "collection_name" => (is => 'ro', isa => 'Str',  required => 1);
has "mail_to"         => (is => 'ro', isa => 'Str',  required => 1);
has "mail_from"       => (is => 'ro', isa => 'Str',  required => 1);
has "mail_devel"      => (is => 'ro', isa => 'Str',  required => 1);
has "mail_admin"      => (is => 'ro', isa => 'Str',  required => 1);
has "mail_bcc"        => (is => 'ro', isa => 'Str',  required => 1);
has "reply_to"        => (is => 'ro', isa => 'Str',  required => 1);
has "test_mode"       => (is => 'ro', isa => 'Bool', required => 1);
has "charts_dir"      => (is => 'ro', isa => 'Str',  required => 1);
has "db"              => (is => 'rw', isa => 'Mentat::Storage::Mongo');
has "stat"            => (is => 'rw', isa => 'Mentat::Stats::Brief');

# Validation hashes for sorting mechanism
my %SORTBYS = (
        'id' => '_id',
        'ts' => 'ts_to',
    );
my %SORTTYPES = (
        'd' => -1,
        'a' => 1,
        'desc' => -1,
        'asc' => 1,
        'descending' => -1,
        'ascending' => 1,
    );

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

# find($$$$$$)
#
#  Usage     : my ($briefs, $count, $count_all, $duration, $query) = $c->model('Briefs')->find($search, $limit, $skip, $sortby, $sort, $reports_dir);
#  Purpose   : Get list of all briefs according to the given parameters
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub find($$$$$$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($search, $limit, $skip, $sortby, $sort, $quick, $reports_dir) = @_;
    die "Brief directory location must be given as argument\n" unless $reports_dir;

    # SEARCH ONLY THROUGH COMPLETE REPORTS
    # Database may contain records like this: {'_id':'SOMEID'}, which work as a
    # lock for ID during brief generation process. So skip those, use 'ts' as
    # marking attribute.
    my $query = {'ts' => {'$exists' => 1}};

    # Initialize the searching condition
    if ($search) {
        my $search_str = $search->{search};
        if ($search_str) {
            my $ip = Value::Convertor->ipv4str_to_int($search_str);

            if ($search_str =~ /^M.*/)           { $query->{'_id'}   = qr/$search_str/; }
            elsif ($ip)                          { $query->{'ip'}    = $search_str; }
            elsif (lc($search_str) eq 'extra')   { $query->{'type'}  = 'extra'; }
            elsif (lc($search_str) eq 'summary') { $query->{'type'}  = 'summary'; }
            elsif (lc($search_str) eq 'unknown') { $query->{'abuse'} = ''; }
            else                                 { $query->{'abuse'} = qr/$search_str/; }
        }

        $query->{ts_from} = {'$lte' => Value::Convertor->datestr_to_unixs($search->{to})}   if $search->{to};
        $query->{ts_to}   = {'$gte' => Value::Convertor->datestr_to_unixs($search->{from})} if $search->{from};
    }

    # Initialize the sorting specification
    my $sort_spec;
    if ($sortby and $sort) {
        $sortby = lc($sortby); $sort = lc($sort);
        die "Invalid sort value '$sortby'" unless $SORTBYS{$sortby};
        die "Invalid sort type '$sort'" unless $SORTTYPES{$sort};
        $sort_spec = {$SORTBYS{$sortby} => $SORTTYPES{$sort}};
    }

    my $resolver = DNS::Resolver->new();
    my $start = time();
    my ($briefs, $count_all) = $self->db->find($query, {message => 0, attachment => 0}, $sort_spec, $limit, $skip);
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return ({}, 0, 0, $duration, $self->db->get_last_filter()) unless $count_all;

    my ($sfrom, $sto, $sts, $count);
    my $result = {};
    foreach my $r (@$briefs) {
        $count++;
        $sfrom = Value::Convertor->unixs_to_ldatestr($r->{ts_from});
        $sto   = Value::Convertor->unixs_to_ldatestr($r->{ts_to});
        $sts   = Value::Convertor->unixs_to_ldatestr($r->{ts});
        $result->{$r->{ts_to}."-".$r->{ts_from}}->{t} = "$sfrom - $sto";
        $result->{$r->{ts_to}."-".$r->{ts_from}}->{tf} = "$sfrom";
        $result->{$r->{ts_to}."-".$r->{ts_from}}->{tt} = "$sto";

        $r->{id}              = $r->{_id};
        $r->{ts_from_s}       = $sfrom;
        $r->{ts_to_s}         = $sto;
        $r->{ts_s}            = $sts;
        $r->{mail_ts}         = (($r->{mail_ts})?Value::Convertor->unixs_to_ldatestr($r->{mail_ts}):'');

        if (exists $r->{list_ips}) {
        $r->{list_ips} = [$r->{list_ips}] unless ref $r->{list_ips} eq 'ARRAY';
        foreach my $ip (@{$r->{list_ips}}) {
            next unless $ip;
            push(@{$r->{list_nodes}}, [$ip, $resolver->resolve_ip($ip)]);
        }
    }

        push(@{$result->{$r->{ts_to}."-".$r->{ts_from}}->{l}}, $r);
    }
    return ($result, $count, $count_all, $duration, $self->db->get_last_filter());
}

# fetch($$)
#
#  Usage     : my ($brief, $count, $duration) = $c->model('Briefs')->fetch($id, $reports_dir);
#  Purpose   : Get one particular brief within the brief storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub fetch($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($brief_id, $reports_dir) = @_;
    die "Brief ID must be given as argument\n" unless $brief_id;
    die "Brief directory location must be given as argument\n" unless $reports_dir;

    my $resolver = DNS::Resolver->new();
    my $start = time();
    my ($briefs, $count) = $self->db->find({'_id' => $brief_id, 'ts' => {'$exists' => 1}});
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return (undef, 0, $duration) unless $count;

    my $brief = shift $briefs;
    $brief->{id}        = $brief->{_id};
    $brief->{downlink}  = (-f $self->brief_data_path($reports_dir, $brief->{_id})) ? 1 : 0;
    $brief->{ts_from_s} = Value::Convertor->unixs_to_ldatestr($brief->{ts_from});
    $brief->{ts_to_s}   = Value::Convertor->unixs_to_ldatestr($brief->{ts_to});
    $brief->{ts_s}      = Value::Convertor->unixs_to_ldatestr($brief->{ts});

    if (exists $brief->{list_ips}) {
        my ($cnt, $ipk);
        $brief->{list_ips} = [$brief->{list_ips}] unless ref $brief->{list_ips} eq 'ARRAY';
        foreach my $ip (@{$brief->{list_ips}}) {
            next unless $ip;
            $ipk = $ip; $ipk =~ s/\./(dot)/g;
            $cnt = $brief->{ips}->{$ipk};
            push(@{$brief->{list_nodes}}, [$ip, $resolver->resolve_ip($ip), $cnt]);
        }
    }

    return ($brief, $count, $duration);
}

# delete($$)
#
#  Usage     : my $rv = $c->model('Briefs')->delete($id, $reports_dir);
#  Purpose   : Delete particular brief from the brief storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub delete($$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($brief_id, $reports_dir) = @_;
    die "Brief ID must be given as argument\n" unless $brief_id;
    die "Brief directory location must be given as argument\n" unless $reports_dir;

    return $self->db->delete($brief_id);
}

# remail($$;$)
#
#  Usage     : my ($rv, $result) = $c->model('Briefs')->remail($id, $reports_dir, $target_email);
#  Purpose   : Resend particular brief via email
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub remail($$;$)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($brief_id, $reports_dir, $target_email) = @_;
    die "Brief ID must be given as argument\n" unless $brief_id;
    die "Brief directory location must be given as argument\n" unless $reports_dir;

    my $mailer = Mentat::Mailer::Report->new(reports_dir => $reports_dir,
                                             mail_from   => $self->mail_from,
                                             mail_devel  => $self->mail_devel,
                                             mail_admin  => $self->mail_admin,
                                             mail_bcc    => $self->mail_bcc,
                                             reply_to    => $self->reply_to,
                                             test_mode   => $self->test_mode);
    my ($briefs, $count) = $self->db->find({'_id' => $brief_id, 'ts' => {'$exists' => 1}});

    return (0, "Brief with ID '$brief_id' was not found") unless $count;

    my $r = shift $briefs;
                                             # $id, $type, $abuse, $to, $message, $attachment, $redirect
    my $to = $target_email || $r->{mail_to};
    my ($rc, $result) = $mailer->email_report($r->{_id}, $r->{type}, $r->{abuse}, $to, $r->{message}, $self->brief_data_file($r->{_id}));

    if ($rc) {
        $r->{mail_ts}  = time;
        $r->{mail_res} = $result;
        my $rv = $self->db->update({'_id' => $r->{_id}}, $r);

        #my $rv = { 'ok' => '1', 'n' => 1, 'connectionId' => 270, 'err' => undef };
        #SUCCESS: { 'ok' => '1', 'n' => 1, 'connectionId' => 270, 'err' => undef }
        #FAILURE: { 'ok' => '1', 'n' => 0, 'connectionId' => 271, 'err' => undef }

        unless ($rv->{ok} and $rv->{n} == 1) {
            $result .= ", however the database could not be updated";
        }
    }

    return ($rc, $result);
}

# calculate($)
#
#  Usage     : my $rv = $c->model('Briefs')->calculate($date, $type, $reports_dir);
#  Purpose   : Calculate statistics for find result
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub calculate($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($date, $type, $reports_dir) = @_;

    my ($th, $tl, $bdate, $btype) = $self->stat->calculate_thresholds($date, $type);

    my $rv = {brief_th => $th, brief_tl => $tl, brief_date => $bdate, brief_type => $btype };
    $rv->{stats_proc} = $self->stat->calc_stats_processing($th, $tl);
    $rv->{stats_rep}  = $self->stat->calc_stats_reporting($th, $tl);
    return $rv;
}

#-------------------------------------------------------------------------------

# BUILD()
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Moose::Manual::Construction(3) for details

sub BUILD {
    my $self = shift;

    # Initialize the internal database connection for whole model
    $self->db(Mentat::Storage::Mongo->new(
                    database => $self->database_name,
                    collection => $self->collection_name));
    $self->stat(Mentat::Stats::Brief->new(charts_dir => $self->charts_dir));

    # If you use 'weak_ref' for moose attribute, previous call will not set the
    # attribute. It is necessary to do it with antermediate variable $self->db($r);
    die "Unable to initialize database connection attribute\n" unless $self->db;
    die "Unable to initialize brief statistics attribute\n" unless $self->stat;
}

# COMPONENT
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Catalyst::Component(3) for details
#
#  FIXME: For some reason config merging does not work without this code present
#         in the package!!!

sub COMPONENT {
    my ($class, $app, $args) = @_;
    $args = $class->merge_config_hashes($class->config, $args);
    #use Data::Dumper;
    #$app->log->debug("X $class: " . Dumper($class->config));
    #$app->log->debug("Y args  : " . Dumper($args));
    #$app->log->debug("Z $app  : " . Dumper($app->config));
    #$app->log->debug("MERGE XY: " . Dumper($args));
    return $class->new($app, $args);
}

1;
